# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import datetime
import warnings

import numpy as np
import pytest

from otx.api.entities.metrics import (
    BarChartInfo,
    BarMetricsGroup,
    ColorPalette,
    CountMetric,
    CurveMetric,
    DateMetric,
    DurationMetric,
    InfoMetric,
    LineChartInfo,
    LineMetricsGroup,
    MatrixChartInfo,
    MatrixMetric,
    MatrixMetricsGroup,
    MultiScorePerformance,
    NullMetric,
    NullPerformance,
    Performance,
    ScoreMetric,
    TextChartInfo,
    TextMetricsGroup,
    VisualizationInfo,
    VisualizationType,
)
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestMetrics:
    @staticmethod
    def mixed_conditions_duration_metric() -> DurationMetric:
        return DurationMetric(name="Mixed conditions metric", hour=0, minute=1, second=2.1)

    @staticmethod
    def matrix_data():
        return np.array([[0, 1, 1], [0, 1, 1], [1, 0, 0]])

    def normalized_matrix_metric(self) -> MatrixMetric:
        return MatrixMetric(name="test matrix", matrix_values=self.matrix_data(), normalize=True)

    @staticmethod
    def normalized_matrix_zero_sum() -> MatrixMetric:
        matrix_data_with_zero_sum = np.array([[0, 0, 0], [0, 1, 1], [1, 0, 0]])
        return MatrixMetric(name="test matrix", matrix_values=matrix_data_with_zero_sum, normalize=True)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_duration_metrics(self):
        """
        <b>Description:</b>
        Check that duration is correctly calculated

        <b>Input data:</b>
        1 hour, 1 minute and 15.4 seconds

        <b>Expected results:</b>
        Test passes if DurationMetrics correctly converts seconds to hours, minutes and seconds

        <b>Steps</b>
        1. Create DurationMetrics
        2. Check hour, minute and second calculated by DurationMetric
        3. Check value returned by get_duration_string method
        """
        hour = 1
        minute = 1
        second = 15.5
        seconds = (hour * 3600) + (minute * 60) + second
        duration_metric = DurationMetric.from_seconds(name="Training duration", seconds=seconds)
        assert duration_metric.hour == hour
        assert duration_metric.minute == minute
        assert duration_metric.second == second
        assert duration_metric.type() == "duration"
        print(duration_metric.get_duration_string())
        # Checking get_duration_string method for 0 specified as DurationMetric parameters
        zero_duration_metric = DurationMetric(name="Zero Duration Metric", hour=0, minute=0, second=0.0)
        assert zero_duration_metric.get_duration_string() == ""
        # Checking get_duration_string method for 1 specified as DurationMetric parameters
        one_duration_metric = DurationMetric(name="One Duration Metric", hour=1, minute=1, second=1.0)
        assert one_duration_metric.get_duration_string() == "1 hour 1 minute 1.00 second"
        # Checking get_duration_string method for value>1 specified as DurationMetric parameters
        more_than_one_duration_metric = DurationMetric(
            name="More than one duration metric", hour=2, minute=3, second=1.1
        )
        assert more_than_one_duration_metric.get_duration_string() == "2 hours 3 minutes 1.10 seconds"
        # Checking get_duration_string method for 0, 1, ">1" values specified as DurationMetric parameters
        mixed_conditions_metric = self.mixed_conditions_duration_metric()
        assert mixed_conditions_metric.get_duration_string() == "1 minute 2.10 seconds"

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_matrix_metric(self):
        """
        <b>Description:</b>
        Check that MatrixMetric correctly normalizes the values in a given matrix

        <b>Input data:</b>
        Three square matrices

        <b>Expected results:</b>
        Test passes if the values of the normalized matrices match the pre-computed matrices

        <b>Steps</b>
        1. Create Matrices
        2. Check normalized matrices against pre-computed matrices
        3. Check positive scenario when row_labels and column_labels parameters specified during MatrixMetric object
        initialization
        4. Check ValueError exception raised when row_labels parameter length is not equal to number of rows of
        MatrixMetric object
        5. Check ValueError exception raised when column_labels parameter length is not equal to number of columns of
        MatrixMetric object
        """
        matrix_metric = self.normalized_matrix_metric()

        required_normalised_matrix_data = np.array([[0, 0.5, 0.5], [0, 0.5, 0.5], [1, 0, 0]])
        assert np.array_equal(required_normalised_matrix_data, matrix_metric.matrix_values)
        assert repr(matrix_metric) == (
            "MatrixMetric(name=`test matrix`, matrix_values=(3x3) matrix, row labels=None, column labels=None)"
        )

        with warnings.catch_warnings():
            # there is a matrix with zero sum in row, so we expect 0/0 division.
            warnings.filterwarnings("ignore", "invalid value encountered in true_divide")
            matrix_data_with_zero_sum = np.array([[0, 0, 0], [0, 1, 1], [1, 0, 0]])
            matrix_metric_with_zero_sum = MatrixMetric(
                name="test matrix",
                matrix_values=matrix_data_with_zero_sum,
                normalize=True,
            )

        required_normalised_matrix_data_with_zero_sum = np.array([[0, 0, 0], [0, 0.5, 0.5], [1, 0, 0]])
        assert np.array_equal(
            required_normalised_matrix_data_with_zero_sum,
            matrix_metric_with_zero_sum.matrix_values,
        )
        # Checking scenario with specified row_labels and column_labels parameters
        matrix_metric_with_labels_name = "MatrixMetric with row and column labels"
        row_labels = ["row_1", "row_2", "row_3"]
        column_labels = ["column_1", "column_2", "column_3"]
        matrix_metric_with_labels = MatrixMetric(
            name=matrix_metric_with_labels_name,
            matrix_values=self.matrix_data(),
            row_labels=row_labels,
            column_labels=column_labels,
        )
        assert matrix_metric_with_labels.name == matrix_metric_with_labels_name
        assert np.array_equal(self.matrix_data(), matrix_metric_with_labels.matrix_values)
        assert matrix_metric_with_labels.row_labels == row_labels
        assert matrix_metric_with_labels.column_labels == column_labels
        assert matrix_metric_with_labels.type() == "matrix"
        assert repr(matrix_metric_with_labels) == (
            "MatrixMetric(name=`MatrixMetric with row and column labels`, matrix_values=(3x3) matrix, "
            "row labels=['row_1', 'row_2', 'row_3'], column labels=['column_1', 'column_2', 'column_3'])"
        )
        # Checking ValueError exception raised when row_labels parameter length not equal to number of rows
        for incorrect_row_labels in (
            ["row_1", "row_2"],
            ["row_1", "row_2", "row_3", "row_4"],
        ):
            with pytest.raises(ValueError):
                MatrixMetric(
                    name="MatrixMetric with incorrect number of row labels",
                    matrix_values=self.matrix_data(),
                    row_labels=incorrect_row_labels,
                )
        # Checking ValueError exception raised when column_labels parameter length not equal to number of columns
        for incorrect_column_labels in (
            ["column_1", "column_2"],
            ["column_1", "column_2", "column_3", "column_4"],
        ):
            with pytest.raises(ValueError):
                MatrixMetric(
                    name="MatrixMetric with incorrect number of column labels",
                    matrix_values=self.matrix_data(),
                    column_labels=incorrect_column_labels,
                )


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestCountMetric:
    @staticmethod
    def count_metric() -> CountMetric:
        return CountMetric(name="Test CountMetric", value=10)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_count_metric(self):
        """
        <b>Description:</b>
        Check CountMetric class

        <b>Input data:</b>
        CountMetric object with specified name and value parameters

        <b>Expected results:</b>
        Test passes if CountMetric object name, value and type attributes return expected values
        """
        for name, value in ["null metric", "positive metric"], [0, 10]:
            count_metric = CountMetric(name=name, value=value)
            assert count_metric.name == name
            assert count_metric.value == value
            assert count_metric.type() == "count"


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestInfoMetric:
    @staticmethod
    def info_metric() -> InfoMetric:
        return InfoMetric(name="Test InfoMetric", value="This Metric is prepared for test purposes")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_info_metric(self):
        """
        <b>Description:</b>
        Check InfoMetric class

        <b>Input data:</b>
        InfoMetric object with specified name and value parameters

        <b>Expected results:</b>
        Test passes if InfoMetric object name, value and type attributes return expected values
        """
        info_metric = self.info_metric()
        assert info_metric.name == "Test InfoMetric"
        assert info_metric.value == "This Metric is prepared for test purposes"
        assert info_metric.type() == "string"


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestDateMetric:
    @staticmethod
    def date_metric_no_date_specified() -> DateMetric:
        return DateMetric(name="DateMetric with not specified date")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_date_metric(self):
        """
        <b>Description:</b>
        Check DateMetric class

        <b>Input data:</b>
        DateMetric object with specified name and date parameters

        <b>Expected results:</b>
        Test passes if DateMetric object name, date and type attributes return expected values

        <b>Steps</b>
        1. Check name, date and type attributes of DateMetric object with not specified date parameter
        2. Check name, date and type attributes of DateMetric object with specified date parameter
        """
        # Check for DateMetric with not specified date parameter
        date_not_specified_metric = self.date_metric_no_date_specified()
        assert date_not_specified_metric.name == "DateMetric with not specified date"
        assert isinstance(date_not_specified_metric.date, datetime.datetime)
        assert date_not_specified_metric.type() == "date"
        # Check for DateMetric with specified date parameter
        date_specified_metric_name = "DateMetric with specified date"
        date_expected = datetime.datetime(year=2020, month=11, day=29, hour=13, minute=25, second=10, microsecond=3)
        date_specified_metric = DateMetric(name=date_specified_metric_name, date=date_expected)
        assert date_specified_metric.name == date_specified_metric_name
        assert date_specified_metric.date == date_expected
        assert date_specified_metric.type() == "date"


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestScoreMetric:
    @staticmethod
    def score_metric() -> ScoreMetric:
        return ScoreMetric(name="Test ScoreMetric", value=2.0)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_score_metric(self):
        """
        <b>Description:</b>
        Check ScoreMetric class

        <b>Input data:</b>
        ScoreMetric object with specified name and value parameters

        <b>Expected results:</b>
        Test passes if ScoreMetric object name, value and type attributes and __eq__ and __repr__ methods return
        expected values

        <b>Steps</b>
        1. Check name, value and type attributes for ScoreMetric object
        2. Check ValueError exception raised when ScoreMetric name parameter is float NaN
        3. Check __eq__ method for ScoreMetric object
        4. Check __repr__ method for ScoreMetric object
        """
        # Checking ScoreMetric object attributes
        score_metric = self.score_metric()
        assert score_metric.name == "Test ScoreMetric"
        assert score_metric.value == 2.0
        assert score_metric.type() == "score"
        # Checking exception raised when value is NaN
        with pytest.raises(ValueError):
            ScoreMetric(name="Test ScoreMetric", value=float("nan"))
        # Checking __eq__ method
        equal_score_metric = ScoreMetric(name="Test ScoreMetric", value=2.0)
        # Checking __eq__ method for equal ScoreMetric objects
        assert score_metric == equal_score_metric
        # Checking __eq__ method for ScoreMetric objects with unequal names
        different_name_score_metric = ScoreMetric(name="Other name ScoreMetric", value=2.0)
        assert score_metric != different_name_score_metric
        # Checking __eq__ method for ScoreMetric objects with unequal values
        different_value_score_metric = ScoreMetric(name="Test ScoreMetric", value=3.4)
        assert score_metric != different_value_score_metric
        # Checking __eq__ method by comparing ScoreMetric object with different type object
        assert score_metric != str
        # Checking __repr__ method
        assert repr(score_metric) == "ScoreMetric(name=`Test ScoreMetric`, score=`2.0`)"


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestCurveMetric:
    @staticmethod
    def ys() -> list:
        return [2.0, 4.1, 3.3, 8.2, 7.1]

    def curve_metric(self) -> CurveMetric:
        xs = [0.0, 0.1, 0.2, 0.3, 0.4]
        return CurveMetric(name="Test CurveMetric", ys=self.ys(), xs=xs)

    def x_not_specified_curve_metric(self) -> CurveMetric:
        return CurveMetric(name="x not specified CurveMetric", ys=self.ys())

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_curve_metric(self):
        """
        <b>Description:</b>
        Check CurveMetric class

        <b>Input data:</b>
        CurveMetric object with specified name, xs and ys parameters

        <b>Expected results:</b>
        Test passes if CurveMetric object has expected attributes and methods

        <b>Steps</b>
        1. Check name, ys, xs and type attributes of CurveMetric object
        2. Check name, ys, xs and type attributes of CurveMetric object with not specified xs initialization parameter
        3. Check ValueError exception raised when length of ys CurveMetric object initialization parameter is not equal
        to length
        of xs parameter
        4. Check __repr__ method for CurveMetric object
        """
        # Checking positive scenario
        curve_metric = self.curve_metric()
        assert curve_metric.name == "Test CurveMetric"
        assert curve_metric.xs == [0.0, 0.1, 0.2, 0.3, 0.4]
        assert curve_metric.ys == self.ys()
        assert curve_metric.type() == "curve"
        # Checking positive scenario with not specified xs parameter
        x_not_specified_curve_metric = self.x_not_specified_curve_metric()
        assert x_not_specified_curve_metric.name == "x not specified CurveMetric"
        assert x_not_specified_curve_metric.xs == [1, 2, 3, 4, 5]
        assert x_not_specified_curve_metric.ys == self.ys()
        assert x_not_specified_curve_metric.type() == "curve"
        # Checking ValueError exception raised when len(ys) != len(xs)
        with pytest.raises(ValueError):
            CurveMetric(name="Negative CurveMetric Scenario", ys=[0.0, 0.1], xs=[1, 2, 3])
        # Checking __repr__ method
        assert repr(curve_metric) == "CurveMetric(name=`Test CurveMetric`, ys=(5 values), xs=(5 values))"


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestNullMetric:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_null_metric(self):
        """
        <b>Description:</b>
        Check NullMetric class

        <b>Input data:</b>
        NullMetric object

        <b>Expected results:</b>
        Test passes if NullMetric object name and type attributes and __repr__ and __eq__ methods return expected values
        """
        # Checking NullMetric attributes
        null_metric = NullMetric()
        assert null_metric.name == "NullMetric"
        assert null_metric.type() == "null"
        # Checking NullMetric __repr__ method
        assert repr(null_metric) == "NullMetric()"
        # Checking NullMetric __eq__ method
        equal_null_metric = NullMetric()
        assert null_metric == equal_null_metric
        # Checking __eq__ method by comparing NullMetric object with different type object
        assert null_metric != TestInfoMetric().info_metric()


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestVisualizationType:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_visualization_type(self):
        """
        <b>Description:</b>
        Check VisualizationType Enum class elements

        <b>Expected results:</b>
        Test passes if VisualizationType Enum class length equal expected value and its elements have expected
        sequence numbers
        """
        assert len(VisualizationType) == 5
        assert VisualizationType.TEXT.value == 0
        assert VisualizationType.RADIAL_BAR.value == 1
        assert VisualizationType.BAR.value == 2
        assert VisualizationType.LINE.value == 3
        assert VisualizationType.MATRIX.value == 4


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestColorPalette:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_color_palette(self):
        """
        <b>Description:</b>
        Check ColorPalette Enum class elements

        <b>Expected results:</b>
        Test passes if ColorPalette Enum class length equal expected value and its elements have expected
        sequence numbers
        """
        assert len(ColorPalette) == 2
        assert ColorPalette.DEFAULT.value == 0
        assert ColorPalette.LABEL.value == 1


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestVisualizationInfo:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_visualization_info(self):
        """
        <b>Description:</b>
        Check VisualizationInfo class

        <b>Input data:</b>
        VisualizationInfo object with specified name, visualisation_type and palette parameters

        <b>Expected results:</b>
        Test passes if VisualizationInfo object name and palette parameters and type and __repr__ methods return
        expected values

        <b>Steps</b>
        1. Check name, type and palette attributes and __repr__ method for VisualizationInfo object with not specified
        palette parameter
        2. Check name, type and palette attributes and __repr__ method for VisualizationInfo object with specified
        palette parameter
        """
        # Checks for not specified palette parameter
        no_palette_specified_name = "No palette specified VisualizationInfo"
        for visualisation_type in VisualizationType:
            visualisation_info = VisualizationInfo(
                name=no_palette_specified_name, visualisation_type=visualisation_type
            )
            assert visualisation_info.name == no_palette_specified_name
            assert visualisation_info.type == visualisation_type
            assert visualisation_info.palette == ColorPalette.DEFAULT
            assert repr(visualisation_info) == (
                f"VisualizationInfo(name='No palette specified VisualizationInfo', "
                f"type='{visualisation_type.name}', palette='DEFAULT')"
            )
        # Check for specified palette parameter
        palette_specified_name = "VisualizationInfo with palette parameter set to LABEL"
        for visualisation_type in VisualizationType:
            visualisation_info = VisualizationInfo(
                name=palette_specified_name,
                visualisation_type=visualisation_type,
                palette=ColorPalette.LABEL,
            )
            assert visualisation_info.name == palette_specified_name
            assert visualisation_info.type == visualisation_type
            assert visualisation_info.palette == ColorPalette.LABEL
            assert repr(visualisation_info) == (
                f"VisualizationInfo(name='VisualizationInfo with palette parameter set "
                f"to LABEL', type='{visualisation_type.name}', palette='LABEL')"
            )


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestTextChartInfo:
    @staticmethod
    def text_chart_info():
        return TextChartInfo("Test TextChartInfo")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_text_chart_info(self):
        """
        <b>Description:</b>
        Check TextChartInfo class

        <b>Input data:</b>
        TextChartInfo object with specified name parameter

        <b>Expected results:</b>
        Test passes if TextChartInfo object name and type attributes and __repr__ method return expected values
        """
        text_chart_info = self.text_chart_info()
        assert text_chart_info.name == "Test TextChartInfo"
        assert text_chart_info.type == VisualizationType.TEXT
        assert repr(text_chart_info) == ("TextChartInfo(name='Test TextChartInfo, " "'type='VisualizationType.TEXT')")


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestLineChartInfo:
    @staticmethod
    def default_parameters_line_chart_info():
        return LineChartInfo("Test default parameters LineChartInfo")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_line_chart_info(self):
        """
        <b>Description:</b>
        Check LineChartInfo class

        <b>Input data:</b>
        LineChartInfo object with specified name, x_axis_label, y_axis_label and palette parameters

        <b>Expected results:</b>
        Test passes if LineChartInfo object name, x_axis_label, y_axis_label, palette and type attributes and
        __repr__ method return expected values

        <b>Steps</b>
        1. Check name, x_axis_label, y_axis_label, palette and type attributes and __repr__ method values for
        LineChartInfo with specified x_axis_label, y_axis_label and palette parameters
        2. Check name, x_axis_label, y_axis_label, palette and type attributes and __repr__ method values for
        LineChartInfo with not specified x_axis_label, y_axis_label and palette parameters
        """
        # Scenario for specified parameters
        line_chart_info_name = "Test LineChartInfo"
        x_axis_label = "Test x-axis label for LineChartInfo"
        y_axis_label = "Test y-axis label for LineChartInfo"
        palette = ColorPalette.LABEL
        parameters_specified_line_chart_info = LineChartInfo(
            name=line_chart_info_name,
            x_axis_label=x_axis_label,
            y_axis_label=y_axis_label,
            palette=palette,
        )
        assert parameters_specified_line_chart_info.name == line_chart_info_name
        assert parameters_specified_line_chart_info.x_axis_label == x_axis_label
        assert parameters_specified_line_chart_info.y_axis_label == y_axis_label
        assert parameters_specified_line_chart_info.palette == palette
        assert parameters_specified_line_chart_info.type == VisualizationType.LINE
        assert repr(parameters_specified_line_chart_info) == (
            "LineChartInfo(name='Test LineChartInfo, 'type='VisualizationType.LINE', x_axis_label='Test x-axis "
            "label for LineChartInfo', y_axis_label='Test y-axis label for LineChartInfo')"
        )
        # Scenario for default parameters
        default_values_line_chart_info = self.default_parameters_line_chart_info()
        assert default_values_line_chart_info.name == "Test default parameters LineChartInfo"
        assert default_values_line_chart_info.x_axis_label == ""
        assert default_values_line_chart_info.y_axis_label == ""
        assert default_values_line_chart_info.palette == ColorPalette.DEFAULT
        assert default_values_line_chart_info.type == VisualizationType.LINE
        assert repr(default_values_line_chart_info) == (
            "LineChartInfo(name='Test default parameters LineChartInfo, 'type='VisualizationType.LINE', "
            "x_axis_label='', y_axis_label='')"
        )


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestBarChartInfo:
    @staticmethod
    def default_parameters_bar_chart_info():
        return BarChartInfo(name="BarChartInfo with default parameters")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_bar_chart_info(self):
        """
        <b>Description:</b>
        Check BarChartInfo class

        <b>Input data:</b>
        BarChartInfo object with specified name, palette and visualization_type parameters

        <b>Expected results:</b>
        Test passes if BarChartInfo object name, type and palette attributes and __repr__ method return expected values

        <b>Steps</b>
        1. Check name, palette and type attributes and __repr__ method values for BarChartInfo object with specified
        palette and visualisation_type parameters
        2. Check name, palette and type attributes and __repr__ method values for BarChartInfo object with not
        specified palette and visualisation_type parameters
        3. Check ValueError exception raised when visualization_type BarChartInfo object initialization parameter is
        not equal to BAR or RADIAL_BAR
        """
        # Scenario for specified parameters
        bar_chart_info_name = "Test BarChartInfo"
        for visualisation_type in [VisualizationType.BAR, VisualizationType.RADIAL_BAR]:
            bar_chart_info = BarChartInfo(
                name=bar_chart_info_name,
                palette=ColorPalette.LABEL,
                visualization_type=visualisation_type,
            )
            assert bar_chart_info.name == bar_chart_info_name
            assert bar_chart_info.palette == ColorPalette.LABEL
            assert bar_chart_info.type == visualisation_type
            assert repr(bar_chart_info) == (f"BarChartInfo(name='Test BarChartInfo', " f"type='{visualisation_type}')")
        # Scenario for default parameters
        default_values_bar_chart_info = self.default_parameters_bar_chart_info()
        assert default_values_bar_chart_info.name == "BarChartInfo with default parameters"
        assert default_values_bar_chart_info.palette == ColorPalette.DEFAULT
        assert default_values_bar_chart_info.type == VisualizationType.BAR
        assert repr(default_values_bar_chart_info) == (
            "BarChartInfo(name='BarChartInfo with default parameters', " "type='VisualizationType.BAR')"
        )
        # Check ValueError exception raised when visualization_type not equal to BAR or RADIAL_BAR
        for visualisation_type in [
            VisualizationType.TEXT,
            VisualizationType.LINE,
            VisualizationType.MATRIX,
        ]:
            with pytest.raises(ValueError):
                BarChartInfo(name=bar_chart_info_name, visualization_type=visualisation_type)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestMatrixChartInfo:
    @staticmethod
    def default_values_matrix_chart_info():
        return MatrixChartInfo("Test MatrixCharInfo with default parameters")

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_matrix_chart_info(self):
        """
        <b>Description:</b>
        Check MatrixChartInfo class

        <b>Input data:</b>
        MatrixChartInfo object with specified name, header, row_header, column_header and palette parameters

        <b>Expected results:</b>
        Test passes if MatrixChartInfo object name, header, row_header, column_header, palette and type attributes and
        __repr__ method return expected values

        <b>Steps</b>
        1. Check name, header, row_header, column_header, palette and type attributes and __repr__ method values for
        MatrixChartInfo object with specified parameters
        2. Check name, header, row_header, column_header, palette and type attributes and __repr__ method values for
        MatrixChartInfo object with default parameters
        """
        # Check for specified parameters
        matrix_chart_info_name = "Test MatrixChartInfo"
        matrix_chart_info_header = "Header of Test MatrixChartInfo"
        matrix_chart_info_row_header = "Specified row header"
        matrix_chart_info_column_header = "Specified column header"
        matrix_chart_info = MatrixChartInfo(
            name=matrix_chart_info_name,
            header=matrix_chart_info_header,
            row_header=matrix_chart_info_row_header,
            column_header=matrix_chart_info_column_header,
            palette=ColorPalette.LABEL,
        )
        assert matrix_chart_info.name == matrix_chart_info_name
        assert matrix_chart_info.header == matrix_chart_info_header
        assert matrix_chart_info.row_header == matrix_chart_info_row_header
        assert matrix_chart_info.column_header == matrix_chart_info_column_header
        assert matrix_chart_info.palette == ColorPalette.LABEL
        assert matrix_chart_info.type == VisualizationType.MATRIX
        assert repr(matrix_chart_info) == (
            "MatrixChartInfo(name='Test MatrixChartInfo', type='VisualizationType.MATRIX', header='Header of Test "
            "MatrixChartInfo', row_header='Specified row header', column_header='Specified column header')"
        )
        # Check for default parameters
        default_parameters_matrix_chart_info = self.default_values_matrix_chart_info()
        assert default_parameters_matrix_chart_info.name == "Test MatrixCharInfo with default parameters"
        assert default_parameters_matrix_chart_info.palette == ColorPalette.DEFAULT
        assert default_parameters_matrix_chart_info.type == VisualizationType.MATRIX
        with pytest.raises(AttributeError):
            default_parameters_matrix_chart_info.header
        with pytest.raises(AttributeError):
            default_parameters_matrix_chart_info.row_header
        with pytest.raises(AttributeError):
            default_parameters_matrix_chart_info.column_header
        with pytest.raises(AttributeError):
            repr(default_parameters_matrix_chart_info)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestMatrixMetricsGroup:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_matrix_metrics_group(self):
        """
        <b>Description:</b>
        Check MatrixMetricsGroup class

        <b>Input data:</b>
        MatrixMetricsGroup object with specified metrics and visualization_info parameters

        <b>Expected results:</b>
        Test passes if MatrixMetricsGroup object metrics and visualization_info attributes return expected values

        <b>Steps</b>
        1. Check metrics and visualization_info attributes for MatrixMetricsGroup object with specified parameters
        2. Check ValueError raised when MatrixMetricsGroup object has metrics parameter equal to None or empty list
        3. Check ValueError raised when MatrixMetricsGroup object has visualization_info parameter equal to None
        """
        # Positive scenario for MatrixMetricsGroup object with specified parameters
        with warnings.catch_warnings():
            # there is a matrix with zero sum in row, so we expect 0/0 division.
            warnings.filterwarnings("ignore", "invalid value encountered in true_divide")
            matrix_metrics = [
                TestMetrics().normalized_matrix_metric(),
                TestMetrics().normalized_matrix_zero_sum(),
            ]
        matrix_chart_info = TestMatrixChartInfo.default_values_matrix_chart_info()
        matrix_metrics_group = MatrixMetricsGroup(metrics=matrix_metrics, visualization_info=matrix_chart_info)
        assert matrix_metrics_group.metrics == matrix_metrics
        assert matrix_metrics_group.visualization_info == matrix_chart_info
        # Negative scenarios for MatrixMetricsGroup object with metrics parameter equal to None or []
        for incorrect_metrics in [None, []]:
            with pytest.raises(ValueError):
                MatrixMetricsGroup(metrics=incorrect_metrics, visualization_info=matrix_chart_info)
        # Negative scenario for MatrixMetricsGroup object with visualization_info parameter equal to None
        with pytest.raises(ValueError):
            MatrixMetricsGroup(metrics=matrix_metrics, visualization_info=None)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestLineMetricsGroup:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_line_metrics_group(self):
        """
        <b>Description:</b>
        Check LineMetricsGroup class

        <b>Input data:</b>
        LineMetricsGroup object with specified metrics and visualization_info parameters

        <b>Expected results:</b>
        Test passes if LineMetricsGroup object metrics and visualization_info attributes return expected values

        <b>Steps</b>
        1. Check metrics and visualization_info attributes for LineMetricsGroup object with specified parameters
        2. Check ValueError raised when LineMetricsGroup object has metrics parameter equal to None or empty tuple
        3. Check ValueError raised when LineMetricsGroup object has visualization_info parameter equal to None
        """
        # Positive scenario for TestLineMetricsGroup object with specified parameters
        curve_metrics = (
            TestCurveMetric().curve_metric(),
            TestCurveMetric().x_not_specified_curve_metric(),
        )
        line_chart_info = TestLineChartInfo().default_parameters_line_chart_info()
        line_metrics_group = LineMetricsGroup(metrics=curve_metrics, visualization_info=line_chart_info)
        assert line_metrics_group.metrics == curve_metrics
        assert line_metrics_group.visualization_info == line_chart_info
        # Negative scenarios for LineMetricsGroup object with metrics parameter equal to None or []
        for incorrect_metrics in [None, ()]:
            with pytest.raises(ValueError):
                LineMetricsGroup(metrics=incorrect_metrics, visualization_info=line_chart_info)
        # Negative scenario for LineMetricsGroup object with visualization_info parameter equal to None
        with pytest.raises(ValueError):
            LineMetricsGroup(metrics=curve_metrics, visualization_info=None)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestBarMetricsGroup:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_bar_metrics_group(self):
        """
        <b>Description:</b>
        Check BarMetricsGroup class

        <b>Input data:</b>
        BarMetricsGroup object with specified metrics and visualization_info parameters

        <b>Expected results:</b>
        Test passes if BarMetricsGroup object metrics and visualization_info attributes return expected values

        <b>Steps</b>
        1. Check metrics and visualization_info attributes for BarMetricsGroup object with specified parameters
        2. Check ValueError raised when BarMetricsGroup object has metrics parameter equal to None or empty list
        3. Check ValueError raised when BarMetricsGroup object has visualization_info parameter equal to None
        """
        # Positive scenario for BarMetricsGroup object with specified parameters
        bar_metrics = [
            TestScoreMetric().score_metric(),
            TestCountMetric().count_metric(),
        ]
        bar_chart_info = TestBarChartInfo().default_parameters_bar_chart_info()
        bar_metrics_group = BarMetricsGroup(metrics=bar_metrics, visualization_info=bar_chart_info)
        assert bar_metrics_group.metrics == bar_metrics
        assert bar_metrics_group.visualization_info == bar_chart_info
        # Negative scenarios for BarMetricsGroup object with metrics parameter equal to None or []
        for incorrect_metrics in [None, []]:
            with pytest.raises(ValueError):
                BarMetricsGroup(metrics=incorrect_metrics, visualization_info=bar_chart_info)
        # Negative scenario for BarMetricsGroup object with visualization_info parameter equal to None
        with pytest.raises(ValueError):
            BarMetricsGroup(metrics=bar_metrics, visualization_info=None)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestTextMetricsGroup:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_text_metrics_group(self):
        """
        <b>Description:</b>
        Check TextMetricsGroup class

        <b>Input data:</b>
        TextMetricsGroup object with specified metrics and visualization_info parameters

        <b>Expected results:</b>
        Test passes if TextMetricsGroup object metrics and visualization_info attributes return expected values

        <b>Steps</b>
        1. Check metrics and visualization_info attributes for TextMetricsGroup object with specified parameters
        2. Check ValueError raised when TextMetricsGroup object has metrics parameter length more than 1
        3. Check ValueError raised when TextMetricsGroup object has metrics parameter equal empty tuple
        4. Check ValueError raised when TextMetricsGroup object has visualization_info parameter equal to None
        """
        # Positive scenario for TextMetricsGroup object with specified parameters
        score_metric = TestScoreMetric().score_metric()
        count_metric = TestCountMetric().count_metric()
        text_chart_info = TestTextChartInfo().text_chart_info()
        for metric in [
            score_metric,
            count_metric,
            TestInfoMetric().info_metric(),
            TestDateMetric().date_metric_no_date_specified(),
            TestMetrics().mixed_conditions_duration_metric(),
        ]:
            text_metric_group = TextMetricsGroup(metrics=[metric], visualization_info=text_chart_info)
            assert text_metric_group.metrics == [metric]
            assert text_metric_group.visualization_info == text_chart_info
        # Negative scenarios for TextMetricsGroup object with metrics parameter length equal to 2
        with pytest.raises(ValueError):
            TextMetricsGroup(metrics=(score_metric, count_metric), visualization_info=text_chart_info)
        # Negative scenarios for TextMetricsGroup object with metrics parameter equal ()
        with pytest.raises(ValueError):
            TextMetricsGroup(metrics=(), visualization_info=text_chart_info)
        # Negative scenario for TextMetricsGroup object with visualization_info parameter equal to None
        with pytest.raises(ValueError):
            TextMetricsGroup(metrics=[score_metric], visualization_info=None)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestPerformance:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_performance(self):
        """
        <b>Description:</b>
        Check Performance class

        <b>Input data:</b>
        Performance object with specified score and dashboard_metrics parameters

        <b>Expected results:</b>
        Test passes if Performance object score and dashboard_metrics attributes and __eq__ and __repr__ method return
        expected values

        <b>Steps</b>
        1. Check score and dashboard_metrics attributes for Performance object with not specified dashboard_metrics
        parameter
        2. Check score and dashboard_metrics attributes for Performance object with specified dashboard_metrics
        parameter
        3. Check __eq__ method for equal Performance objects, Performance objects with different dashboard_metrics
        attributes, Performance objects with different score attributes
        4. Check __repr__ method
        5. Check ValueError exception raised when score attributes type not equal to ScoreMetric
        """
        # Positive scenario for Performance object with default parameters
        score_metric = TestScoreMetric().score_metric()
        default_parameters_performance = Performance(score_metric)
        assert default_parameters_performance.score == score_metric
        assert default_parameters_performance.dashboard_metrics == []
        # Positive scenario for Performance object with specified dashboard_metrics  parameter
        # Preparing dashboard metrics list
        with warnings.catch_warnings():
            # there is a matrix with zero sum in row, so we expect 0/0 division.
            warnings.filterwarnings("ignore", "invalid value encountered in true_divide")
            matrix_metrics = [
                TestMetrics().normalized_matrix_metric(),
                TestMetrics().normalized_matrix_zero_sum(),
            ]
        matrix_chart_info = TestMatrixChartInfo.default_values_matrix_chart_info()
        matrix_metrics_group = MatrixMetricsGroup(metrics=matrix_metrics, visualization_info=matrix_chart_info)
        curve_metrics = (
            TestCurveMetric().curve_metric(),
            TestCurveMetric().x_not_specified_curve_metric(),
        )
        line_chart_info = TestLineChartInfo().default_parameters_line_chart_info()
        line_metrics_group = LineMetricsGroup(metrics=curve_metrics, visualization_info=line_chart_info)
        bar_metrics = [
            TestScoreMetric().score_metric(),
            TestCountMetric().count_metric(),
        ]
        bar_chart_info = TestBarChartInfo().default_parameters_bar_chart_info()
        bar_metrics_group = BarMetricsGroup(metrics=bar_metrics, visualization_info=bar_chart_info)
        text_score_metric = TestScoreMetric().score_metric()
        text_chart_info = TestTextChartInfo().text_chart_info()
        text_metric_group = TextMetricsGroup(metrics=[text_score_metric], visualization_info=text_chart_info)
        dashboard_metrics = [
            matrix_metrics_group,
            line_metrics_group,
            bar_metrics_group,
            text_metric_group,
        ]
        # Checking Performance attributes
        specified_parameters_performance = Performance(score=score_metric, dashboard_metrics=dashboard_metrics)
        assert specified_parameters_performance.score == score_metric
        assert specified_parameters_performance.dashboard_metrics == dashboard_metrics
        # Checking __eq__ method
        equal_default_parameters_performance = Performance(score_metric)
        assert default_parameters_performance == equal_default_parameters_performance
        different_metrics_performance = Performance(score_metric, [matrix_metrics_group])
        assert default_parameters_performance == different_metrics_performance
        unequal_score_metric = ScoreMetric(name="Unequal ScoreMetric", value=1.0)
        assert default_parameters_performance != Performance(unequal_score_metric)
        assert default_parameters_performance != str
        # Checking __repr__ method
        assert repr(default_parameters_performance) == "Performance(score: 2.0, dashboard: (0 metric groups))"
        assert repr(specified_parameters_performance) == "Performance(score: 2.0, dashboard: (4 metric groups))"
        # Checking ValueError exception raised when score parameter not ScoreMetric class
        count_metric = TestCountMetric().count_metric()
        with pytest.raises(ValueError):
            Performance(count_metric)


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestNullPerformance:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_null_performance(self):
        """
        <b>Description:</b>
        Check NullPerformance class

        <b>Input data:</b>
        NullPerformance object

        <b>Expected results:</b>
        Test passes if NullPerformance object score and dashboard_metrics attributes and __repr__ and __eq__ methods
        return expected values

        <b>Steps</b>
        1. Check NullPerformance object score and dashboard_metrics attributes
        2. Check NullPerformance object __repr__ method
        3. Check NullPerformance object __eq__ method
        """
        # Checking NullPerformance score and dashboard_metrics attributes
        null_performance = NullPerformance()
        assert null_performance.score == ScoreMetric(name="Null score", value=0.0)
        assert null_performance.dashboard_metrics == []
        # Checking NullPerformance __repr__ method
        assert repr(null_performance) == "NullPerformance()"
        # Checking __eq__ method for equal NullPerformance objects
        equal_null_performance = NullPerformance()
        assert null_performance == equal_null_performance
        # Checking NullPerformance __eq__ method by comparing with Performance object
        score_metric = TestScoreMetric().score_metric()
        performance = Performance(score_metric)
        assert null_performance != performance


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestMultiScorePerformance:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_multi_score_performance(self):
        """
        <b>Description:</b>
        Check MultiScorePerformance class

        <b>Input data:</b>
        MultiScorePerformance object with specified score

        <b>Expected results:</b>
        Test passes if MultiScorePerformance object score attribute and __eq__ and __repr__ method return
        expected values

        <b>Steps</b>
        1. Check primary and additional score attributes for MultiScorePerformance object
        2. Check primary and additional score attributes for MultiScorePerformance object when only primary score is
        passed
        3. Check primary and additional score attributes for MultiScorePerformance object when only additional score is
        passed
        4. Check __eq__ method for equal and unequal Performance objects
        5. Check __repr__ method
        """
        # Positive scenario for Performance object with default parameters
        primary_score = TestScoreMetric().score_metric()
        additional_score = TestScoreMetric().score_metric()
        default_parameters_performance = MultiScorePerformance(primary_score, [additional_score])
        assert default_parameters_performance.score == primary_score
        assert default_parameters_performance.primary_score == primary_score
        assert default_parameters_performance.additional_scores == [additional_score]
        assert default_parameters_performance.dashboard_metrics == []
        # Positive scenario for Performance object with only primary metric
        only_primary_performance = MultiScorePerformance(primary_score)
        assert only_primary_performance.score == primary_score
        assert only_primary_performance.primary_score == primary_score
        assert only_primary_performance.additional_scores == []
        assert only_primary_performance.dashboard_metrics == []
        # Positive scenario for Performance object with only additional metric
        only_additional_performance = MultiScorePerformance(additional_scores=[additional_score])
        assert only_additional_performance.score == additional_score
        assert only_additional_performance.primary_score is None
        assert only_additional_performance.additional_scores == [additional_score]
        assert only_additional_performance.dashboard_metrics == []
        # Checking __eq__ method
        equal_default_parameters_performance = MultiScorePerformance(primary_score, [additional_score])
        assert default_parameters_performance == equal_default_parameters_performance
        assert default_parameters_performance != only_primary_performance
        # Checking __repr__ method
        assert (
            repr(default_parameters_performance)
            == "MultiScorePerformance(score: 2.0, primary_metric: ScoreMetric(name=`Test ScoreMetric`, score=`2.0`), "
            "additional_metrics: (1 metrics), dashboard: (0 metric groups))"
        )



import pytest
from torch.utils.data import Dataset

from otx.algorithms.common.adapters.torch.dataloaders.samplers import BalancedSampler
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestBalancedSampler:
    @pytest.fixture(autouse=True)
    def setup(self):
        class MockDataset(Dataset):
            def __init__(self):
                self.img_indices = {"foo": list(range(0, 6)), "bar": list(range(6, 10))}

            def __len__(self):
                return 10

        self.mock_dataset = MockDataset()

    @e2e_pytest_unit
    def test_sampler_iter(self):
        sampler = BalancedSampler(self.mock_dataset, 4)
        sampler_iter = iter(sampler)
        count = 0

        for _ in sampler_iter:
            count += 1

        assert count == len(sampler)

    @e2e_pytest_unit
    def test_sampler_iter_with_multiple_replicas(self):
        sampler = BalancedSampler(self.mock_dataset, 4, num_replicas=2)
        sampler_iter = iter(sampler)
        count = 0

        for _ in sampler_iter:
            count += 1

        assert count == len(sampler)



"""Base configurer for mmdet config."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import importlib
import json
import os
from typing import Any, Dict

import numpy as np
import torch
from mmcv.runner import CheckpointLoader
from mmcv.utils import Config, ConfigDict
from torch import distributed as dist

from otx.algorithms.common.adapters.mmcv.utils import (
    align_data_config_with_recipe,
    build_dataloader,
    build_dataset,
    patch_adaptive_interval_training,
    patch_default_config,
    patch_early_stopping,
    patch_fp16,
    patch_persistent_workers,
    patch_runner,
)
from otx.algorithms.common.adapters.mmcv.utils.config_utils import (
    recursively_update_cfg,
    update_or_add_custom_hook,
)
from otx.algorithms.common.utils.logger import get_logger
from otx.algorithms.detection.adapters.mmdet.utils import (
    patch_datasets,
    patch_evaluation,
)

logger = get_logger()


# pylint: disable=too-many-public-methods
class DetectionConfigurer:
    """Patch config to support otx train."""

    def __init__(self):
        self.task_adapt_type = None
        self.task_adapt_op = "REPLACE"
        self.org_model_classes = []
        self.model_classes = []
        self.data_classes = []

    # pylint: disable=too-many-arguments
    def configure(
        self,
        cfg,
        model_ckpt,
        data_cfg,
        training=True,
        subset="train",
        ir_options=None,
        data_classes=None,
        model_classes=None,
    ):
        """Create MMCV-consumable config from given inputs."""
        logger.info(f"configure!: training={training}")

        self.configure_base(cfg, data_cfg, data_classes, model_classes)
        self.configure_device(cfg, training)
        self.configure_model(cfg, ir_options)
        self.configure_ckpt(cfg, model_ckpt)
        self.configure_data(cfg, training, data_cfg)
        self.configure_regularization(cfg, training)
        self.configure_task(cfg, training)
        self.configure_hook(cfg)
        self.configure_samples_per_gpu(cfg, subset)
        self.configure_fp16_optimizer(cfg)
        self.configure_compat_cfg(cfg)
        return cfg

    def configure_base(self, cfg, data_cfg, data_classes, model_classes):
        """Basic configuration work for recipe.

        Patchings in this function are handled task level previously
        This function might need to be re-orgianized
        """

        options_for_patch_datasets = {"type": "OTXDetDataset"}

        patch_default_config(cfg)
        patch_runner(cfg)
        patch_datasets(
            cfg,
            **options_for_patch_datasets,
        )  # for OTX compatibility
        patch_evaluation(cfg)  # for OTX compatibility
        patch_fp16(cfg)
        patch_adaptive_interval_training(cfg)
        patch_early_stopping(cfg)
        patch_persistent_workers(cfg)

        if data_cfg is not None:
            align_data_config_with_recipe(data_cfg, cfg)

        # update model config -> model label schema
        cfg["model_classes"] = model_classes
        if data_classes is not None:
            train_data_cfg = self.get_data_cfg(data_cfg, "train")
            train_data_cfg["data_classes"] = data_classes
            new_classes = np.setdiff1d(data_classes, model_classes).tolist()
            train_data_cfg["new_classes"] = new_classes

    def configure_model(self, cfg, ir_options):  # noqa: C901
        """Patch config's model.

        Change model type to super type
        Patch for OMZ backbones
        """

        if ir_options is None:
            ir_options = {"ir_model_path": None, "ir_weight_path": None, "ir_weight_init": False}

        cfg.model_task = cfg.model.pop("task", "detection")
        if cfg.model_task != "detection":
            raise ValueError(f"Given cfg ({cfg.filename}) is not supported by detection recipe")

        super_type = cfg.model.pop("super_type", None)
        if super_type:
            cfg.model.arch_type = cfg.model.type
            cfg.model.type = super_type

        # OV-plugin
        ir_model_path = ir_options.get("ir_model_path")
        if ir_model_path:

            def is_mmov_model(key, value):
                if key == "type" and value.startswith("MMOV"):
                    return True
                return False

            ir_weight_path = ir_options.get("ir_weight_path", None)
            ir_weight_init = ir_options.get("ir_weight_init", False)
            recursively_update_cfg(
                cfg,
                is_mmov_model,
                {"model_path": ir_model_path, "weight_path": ir_weight_path, "init_weight": ir_weight_init},
            )

    def configure_data(self, cfg, training, data_cfg):  # noqa: C901
        """Patch cfg.data.

        Merge cfg and data_cfg
        Match cfg.data.train.type to super_type
        Patch for unlabeled data path ==> This may be moved to SemiDetectionConfigurer
        """
        if data_cfg:
            cfg.merge_from_dict(data_cfg)

        def configure_split(target):
            def update_transform(opt, pipeline, idx, transform):
                if isinstance(opt, dict):
                    if "_delete_" in opt.keys() and opt.get("_delete_", False):
                        # if option include _delete_=True, remove this transform from pipeline
                        logger.info(f"configure_data: {transform['type']} is deleted")
                        del pipeline[idx]
                        return
                    logger.info(f"configure_data: {transform['type']} is updated with {opt}")
                    transform.update(**opt)

            # pylint: disable=too-many-branches, too-many-nested-blocks
            def update_config(src, pipeline_options):
                logger.info(f"update_config() {pipeline_options}")
                if src.get("pipeline") is not None or (
                    src.get("dataset") is not None and src.get("dataset").get("pipeline") is not None
                ):
                    if src.get("pipeline") is not None:
                        pipeline = src.get("pipeline", None)
                    else:
                        pipeline = src.get("dataset").get("pipeline")
                    if isinstance(pipeline, list):
                        for idx, transform in enumerate(pipeline):
                            for opt_key, opt in pipeline_options.items():
                                if transform["type"] == opt_key:
                                    update_transform(opt, pipeline, idx, transform)
                    elif isinstance(pipeline, dict):
                        for _, pipe in pipeline.items():
                            for idx, transform in enumerate(pipe):
                                for opt_key, opt in pipeline_options.items():
                                    if transform["type"] == opt_key:
                                        update_transform(opt, pipe, idx, transform)
                    else:
                        raise NotImplementedError(f"pipeline type of {type(pipeline)} is not supported")
                else:
                    logger.info("no pipeline in the data split")

            split = cfg.data.get(target)
            if split is not None:
                if isinstance(split, list):
                    for sub_item in split:
                        update_config(sub_item, pipeline_options)
                elif isinstance(split, dict):
                    update_config(split, pipeline_options)
                else:
                    logger.warning(f"type of split '{target}'' should be list or dict but {type(split)}")

        logger.info("configure_data()")
        logger.debug(f"[args] {cfg.data}")
        pipeline_options = cfg.data.pop("pipeline_options", None)
        if pipeline_options is not None and isinstance(pipeline_options, dict):
            configure_split("train")
            configure_split("val")
            if not training:
                configure_split("test")
            configure_split("unlabeled")
        super_type = cfg.data.train.pop("super_type", None)
        if super_type:
            cfg.data.train.org_type = cfg.data.train.type
            cfg.data.train.type = super_type

    def configure_regularization(self, cfg, training):  # noqa: C901
        """Patch regularization parameters."""
        if training:
            if cfg.model.get("l2sp_weight", 0.0) > 0.0:
                logger.info("regularization config!!!!")

                # Checkpoint
                l2sp_ckpt = cfg.model.get("l2sp_ckpt", None)
                if l2sp_ckpt is None:
                    if "pretrained" in cfg.model:
                        l2sp_ckpt = cfg.model.pretrained
                    if cfg.load_from:
                        l2sp_ckpt = cfg.load_from
                cfg.model.l2sp_ckpt = l2sp_ckpt

                # Disable weight decay
                if "weight_decay" in cfg.optimizer:
                    cfg.optimizer.weight_decay = 0.0

    def configure_task(self, cfg, training):
        """Patch config to support training algorithm."""
        if "task_adapt" in cfg:
            logger.info(f"task config!!!!: training={training}")
            self.task_adapt_type = cfg["task_adapt"].get("type", None)
            self.task_adapt_op = cfg["task_adapt"].get("op", "REPLACE")
            self.configure_classes(cfg)

            if self.data_classes != self.model_classes:
                self.configure_task_data_pipeline(cfg)
            # TODO[JAEGUK]: configure_anchor is not working
            if cfg["task_adapt"].get("use_mpa_anchor", False):
                self.configure_anchor(cfg)
            if self.task_adapt_type == "mpa":
                self.configure_bbox_head(cfg)
                self.configure_ema(cfg)
            else:
                src_data_cfg = self.get_data_cfg(cfg, "train")
                src_data_cfg.pop("old_new_indices", None)

    # pylint: disable=too-many-branches
    def configure_classes(self, cfg):
        """Patch classes for model and dataset."""
        org_model_classes = self.get_model_classes(cfg)
        data_classes = self.get_data_classes(cfg)

        # Model classes
        if self.task_adapt_op == "REPLACE":
            if len(data_classes) == 0:
                model_classes = org_model_classes.copy()
            else:
                model_classes = data_classes.copy()
        elif self.task_adapt_op == "MERGE":
            model_classes = org_model_classes + [cls for cls in data_classes if cls not in org_model_classes]
        else:
            raise KeyError(f"{self.task_adapt_op} is not supported for task_adapt options!")

        if self.task_adapt_type == "mpa":
            data_classes = model_classes
        cfg.task_adapt.final = model_classes
        cfg.model.task_adapt = ConfigDict(
            src_classes=org_model_classes,
            dst_classes=model_classes,
        )

        # Model architecture
        head_names = ("mask_head", "bbox_head", "segm_head")
        num_classes = len(model_classes)
        if "roi_head" in cfg.model:
            # For Faster-RCNNs
            for head_name in head_names:
                if head_name in cfg.model.roi_head:
                    if isinstance(cfg.model.roi_head[head_name], list):
                        for head in cfg.model.roi_head[head_name]:
                            head.num_classes = num_classes
                    else:
                        cfg.model.roi_head[head_name].num_classes = num_classes
        else:
            # For other architectures (including SSD)
            for head_name in head_names:
                if head_name in cfg.model:
                    cfg.model[head_name].num_classes = num_classes

        # Eval datasets
        if cfg.get("task", "detection") == "detection":
            eval_types = ["val", "test"]
            for eval_type in eval_types:
                if cfg.data[eval_type]["type"] == "TaskAdaptEvalDataset":
                    cfg.data[eval_type]["model_classes"] = model_classes
                else:
                    # Wrap original dataset config
                    org_type = cfg.data[eval_type]["type"]
                    cfg.data[eval_type]["type"] = "TaskAdaptEvalDataset"
                    cfg.data[eval_type]["org_type"] = org_type
                    cfg.data[eval_type]["model_classes"] = model_classes

        self.org_model_classes = org_model_classes
        self.model_classes = model_classes
        self.data_classes = data_classes

    def configure_task_data_pipeline(self, cfg):
        """Trying to alter class indices of training data according to model class order."""
        tr_data_cfg = self.get_data_cfg(cfg, "train")
        class_adapt_cfg = dict(type="AdaptClassLabels", src_classes=self.data_classes, dst_classes=self.model_classes)
        pipeline_cfg = tr_data_cfg.pipeline
        for i, operation in enumerate(pipeline_cfg):
            if operation["type"] == "LoadAnnotations":  # insert just after this operation
                op_next_ann = pipeline_cfg[i + 1] if i + 1 < len(pipeline_cfg) else {}
                if op_next_ann.get("type", "") == class_adapt_cfg["type"]:
                    op_next_ann.update(class_adapt_cfg)
                else:
                    pipeline_cfg.insert(i + 1, class_adapt_cfg)
                break

    def configure_anchor(self, cfg):
        """Patch anchor settings for single stage detector."""
        if cfg.model.type in ["SingleStageDetector", "CustomSingleStageDetector"]:
            anchor_cfg = cfg.model.bbox_head.anchor_generator
            if anchor_cfg.type == "SSDAnchorGeneratorClustered":
                cfg.model.bbox_head.anchor_generator.pop("input_size", None)

    def configure_bbox_head(self, cfg):
        """Patch bbox head in detector for class incremental learning.

        Most of patching are related with hyper-params in focal loss
        """
        if cfg.get("task", "detection") == "detection":
            bbox_head = cfg.model.bbox_head
        else:
            bbox_head = cfg.model.roi_head.bbox_head

        alpha, gamma = 0.25, 2.0
        if bbox_head.type in ["SSDHead", "CustomSSDHead"]:
            gamma = 1 if cfg["task_adapt"].get("efficient_mode", False) else 2
            bbox_head.type = "CustomSSDHead"
            bbox_head.loss_cls = ConfigDict(
                type="FocalLoss",
                loss_weight=1.0,
                gamma=gamma,
                reduction="none",
            )
        elif bbox_head.type in ["ATSSHead"]:
            gamma = 3 if cfg["task_adapt"].get("efficient_mode", False) else 4.5
            bbox_head.loss_cls.gamma = gamma
        elif bbox_head.type in ["VFNetHead", "CustomVFNetHead"]:
            alpha = 0.75
            gamma = 1 if cfg["task_adapt"].get("efficient_mode", False) else 2
        # TODO Move this part
        # This is not related with patching bbox head
        elif bbox_head.type in ["YOLOXHead", "CustomYOLOXHead"]:
            if cfg.data.train.type == "MultiImageMixDataset":
                self.add_yolox_hooks(cfg)

        if cfg.get("ignore", False):
            bbox_head.loss_cls = ConfigDict(
                type="CrossSigmoidFocalLoss",
                use_sigmoid=True,
                num_classes=len(self.model_classes),
                alpha=alpha,
                gamma=gamma,
            )

    @staticmethod
    def configure_ema(cfg):
        """Patch ema settings."""
        adaptive_ema = cfg.get("adaptive_ema", {})
        if adaptive_ema:
            update_or_add_custom_hook(
                cfg,
                ConfigDict(
                    type="CustomModelEMAHook",
                    priority="ABOVE_NORMAL",
                    resume_from=cfg.get("resume_from"),
                    **adaptive_ema,
                ),
            )
        else:
            update_or_add_custom_hook(
                cfg,
                ConfigDict(type="EMAHook", priority="ABOVE_NORMAL", resume_from=cfg.get("resume_from"), momentum=0.1),
            )

    @staticmethod
    def add_yolox_hooks(cfg):
        """Update yolox hooks."""
        update_or_add_custom_hook(
            cfg,
            ConfigDict(
                type="YOLOXModeSwitchHook",
                num_last_epochs=15,
                priority=48,
            ),
        )
        update_or_add_custom_hook(
            cfg,
            ConfigDict(
                type="SyncNormHook",
                num_last_epochs=15,
                interval=1,
                priority=48,
            ),
        )

    # Functions below are come from base stage
    def configure_ckpt(self, cfg, model_ckpt):
        """Patch checkpoint path for pretrained weight.

        Replace cfg.load_from to model_ckpt
        Replace cfg.load_from to pretrained
        Replace cfg.resume_from to cfg.load_from
        """
        if model_ckpt:
            cfg.load_from = self.get_model_ckpt(model_ckpt)
        if cfg.get("resume", False):
            cfg.resume_from = cfg.load_from

    @staticmethod
    def get_model_ckpt(ckpt_path, new_path=None):
        """Get pytorch model weights."""
        ckpt = CheckpointLoader.load_checkpoint(ckpt_path, map_location="cpu")
        if "model" in ckpt:
            ckpt = ckpt["model"]
            if not new_path:
                new_path = ckpt_path[:-3] + "converted.pth"
            torch.save(ckpt, new_path)
            return new_path
        return ckpt_path

    @staticmethod
    def get_model_classes(cfg):
        """Extract trained classes info from checkpoint file.

        MMCV-based models would save class info in ckpt['meta']['CLASSES']
        For other cases, try to get the info from cfg.model.classes (with pop())
        - Which means that model classes should be specified in model-cfg for
          non-MMCV models (e.g. OMZ models)
        """

        def get_model_meta(cfg):
            ckpt_path = cfg.get("load_from", None)
            meta = {}
            if ckpt_path:
                ckpt = CheckpointLoader.load_checkpoint(ckpt_path, map_location="cpu")
                meta = ckpt.get("meta", {})
            return meta

        def read_label_schema(ckpt_path, name_only=True, file_name="label_schema.json"):
            serialized_label_schema = []
            if any(ckpt_path.endswith(extension) for extension in (".xml", ".bin", ".pth")):
                label_schema_path = os.path.join(os.path.dirname(ckpt_path), file_name)
                if os.path.exists(label_schema_path):
                    with open(label_schema_path, encoding="UTF-8") as read_file:
                        serialized_label_schema = json.load(read_file)
            if serialized_label_schema:
                if name_only:
                    all_classes = [labels["name"] for labels in serialized_label_schema["all_labels"].values()]
                else:
                    all_classes = serialized_label_schema
            else:
                all_classes = []
            return all_classes

        classes = []
        meta = get_model_meta(cfg)
        # for MPA classification legacy compatibility
        classes = meta.get("CLASSES", [])
        classes = meta.get("classes", classes)
        if classes is None:
            classes = []

        if len(classes) == 0:
            ckpt_path = cfg.get("load_from", None)
            if ckpt_path:
                classes = read_label_schema(ckpt_path)
        if len(classes) == 0:
            classes = cfg.model.pop("classes", cfg.pop("model_classes", []))
        return classes

    def get_data_classes(self, cfg):
        """Get data classes from train cfg."""
        data_classes = []
        train_cfg = self.get_data_cfg(cfg, "train")
        if "data_classes" in train_cfg:
            data_classes = list(train_cfg.pop("data_classes", []))
        elif "classes" in train_cfg:
            data_classes = list(train_cfg.classes)
        return data_classes

    @staticmethod
    def get_data_cfg(cfg, subset):
        """Get subset's data cfg."""
        assert subset in ["train", "val", "test"], f"Unknown subset:{subset}"
        if "dataset" in cfg.data[subset]:  # Concat|RepeatDataset
            dataset = cfg.data[subset].dataset
            while hasattr(dataset, "dataset"):
                dataset = dataset.dataset
            return dataset
        return cfg.data[subset]

    @staticmethod
    def configure_hook(cfg):
        """Update cfg.custom_hooks based on cfg.custom_hook_options."""

        def update_hook(opt, custom_hooks, idx, hook):
            """Delete of update a custom hook."""
            if isinstance(opt, dict):
                if opt.get("_delete_", False):
                    # if option include _delete_=True, remove this hook from custom_hooks
                    logger.info(f"configure_hook: {hook['type']} is deleted")
                    del custom_hooks[idx]
                else:
                    logger.info(f"configure_hook: {hook['type']} is updated with {opt}")
                    hook.update(**opt)

        hook_cfg = ConfigDict(type="LoggerReplaceHook")
        update_or_add_custom_hook(cfg, hook_cfg)

        custom_hook_options = cfg.pop("custom_hook_options", {})
        # logger.info(f"configure_hook() {cfg.get('custom_hooks', [])} <- {custom_hook_options}")
        custom_hooks = cfg.get("custom_hooks", [])
        for idx, hook in enumerate(custom_hooks):
            for opt_key, opt in custom_hook_options.items():
                if hook["type"] == opt_key:
                    update_hook(opt, custom_hooks, idx, hook)

    def configure_device(self, cfg, training):
        """Setting device for training and inference."""
        cfg.distributed = False
        if torch.distributed.is_initialized():
            cfg.gpu_ids = [int(os.environ["LOCAL_RANK"])]
            if training:  # TODO multi GPU is available only in training. Evaluation needs to be supported later.
                cfg.distributed = True
                self.configure_distributed(cfg)
        elif "gpu_ids" not in cfg:
            cfg.gpu_ids = range(1)

        # consider "cuda" and "cpu" device only
        if not torch.cuda.is_available():
            cfg.device = "cpu"
            cfg.gpu_ids = range(-1, 0)
        else:
            cfg.device = "cuda"

    def configure_samples_per_gpu(
        self,
        cfg: Config,
        subset: str,
    ):
        """Settings samples_per_gpu for training and inference."""

        dataloader_cfg = cfg.data.get(f"{subset}_dataloader", ConfigDict())
        samples_per_gpu = dataloader_cfg.get("samples_per_gpu", cfg.data.get("samples_per_gpu", 1))

        data_cfg = self.get_data_cfg(cfg, subset)
        if data_cfg.get("otx_dataset") is not None:
            dataset_len = len(data_cfg.otx_dataset)

            if getattr(cfg, "distributed", False):
                dataset_len = dataset_len // dist.get_world_size()

            # set batch size as a total dataset
            # if it is smaller than total dataset
            if dataset_len < samples_per_gpu:
                dataloader_cfg.samples_per_gpu = dataset_len

            # drop the last batch if the last batch size is 1
            # batch size of 1 is a runtime error for training batch normalization layer
            if subset in ("train", "unlabeled") and dataset_len % samples_per_gpu == 1:
                dataloader_cfg["drop_last"] = True

            cfg.data[f"{subset}_dataloader"] = dataloader_cfg

    @staticmethod
    def configure_fp16_optimizer(cfg: Config):
        """Configure Fp16OptimizerHook and Fp16SAMOptimizerHook."""

        fp16_config = cfg.pop("fp16", None)
        if fp16_config is not None:
            optim_type = cfg.optimizer_config.get("type", "OptimizerHook")
            opts: Dict[str, Any] = dict(
                distributed=getattr(cfg, "distributed", False),
                **fp16_config,
            )
            if optim_type == "SAMOptimizerHook":
                opts["type"] = "Fp16SAMOptimizerHook"
            elif optim_type == "OptimizerHook":
                opts["type"] = "Fp16OptimizerHook"
            else:
                # does not support optimizerhook type
                # let mm library handle it
                cfg.fp16 = fp16_config
                opts = dict()
            cfg.optimizer_config.update(opts)

    @staticmethod
    def configure_distributed(cfg):
        """Patching for distributed training."""
        if hasattr(cfg, "dist_params") and cfg.dist_params.get("linear_scale_lr", False):
            new_lr = len(cfg.gpu_ids) * cfg.optimizer.lr
            logger.info(
                f"enabled linear scaling rule to the learning rate. \
                changed LR from {cfg.optimizer.lr} to {new_lr}"
            )
            cfg.optimizer.lr = new_lr

    @staticmethod
    def configure_compat_cfg(
        cfg: Config,
    ):
        """Modify config to keep the compatibility."""

        def _configure_dataloader(cfg):
            global_dataloader_cfg = {}
            global_dataloader_cfg.update(
                {
                    k: cfg.data.pop(k)
                    for k in list(cfg.data.keys())
                    if k
                    not in [
                        "train",
                        "val",
                        "test",
                        "unlabeled",
                        "train_dataloader",
                        "val_dataloader",
                        "test_dataloader",
                        "unlabeled_dataloader",
                    ]
                }
            )

            for subset in ["train", "val", "test", "unlabeled"]:
                if subset not in cfg.data:
                    continue
                dataloader_cfg = cfg.data.get(f"{subset}_dataloader", None)
                if dataloader_cfg is None:
                    raise AttributeError(f"{subset}_dataloader is not found in config.")
                dataloader_cfg = Config(cfg_dict={**global_dataloader_cfg, **dataloader_cfg})
                cfg.data[f"{subset}_dataloader"] = dataloader_cfg

        _configure_dataloader(cfg)


class IncrDetectionConfigurer(DetectionConfigurer):
    """Patch config to support incremental learning for object detection."""

    def configure_task(self, cfg, training):
        """Patch config to support incremental learning."""
        super().configure_task(cfg, training)
        if "task_adapt" in cfg and self.task_adapt_type == "mpa":
            self.configure_task_adapt_hook(cfg)

    def configure_task_adapt_hook(self, cfg):
        """Add TaskAdaptHook for sampler."""
        sampler_flag = True
        if len(set(self.org_model_classes) & set(self.model_classes)) == 0 or set(self.org_model_classes) == set(
            self.model_classes
        ):
            sampler_flag = False
        update_or_add_custom_hook(
            cfg,
            ConfigDict(
                type="TaskAdaptHook",
                src_classes=self.org_model_classes,
                dst_classes=self.model_classes,
                model_type=cfg.model.type,
                sampler_flag=sampler_flag,
                efficient_mode=cfg["task_adapt"].get("efficient_mode", False),
            ),
        )


class SemiSLDetectionConfigurer(DetectionConfigurer):
    """Patch config to support semi supervised learning for object detection."""

    def configure_data(self, cfg, training, data_cfg):
        """Patch cfg.data."""
        super().configure_data(cfg, training, data_cfg)
        # Set unlabeled data hook
        if training:
            if cfg.data.get("unlabeled", False) and cfg.data.unlabeled.get("otx_dataset", False):
                if len(cfg.data.unlabeled.get("pipeline", [])) == 0:
                    cfg.data.unlabeled.pipeline = cfg.data.train.pipeline.copy()
                self.configure_unlabeled_dataloader(cfg)

    def configure_task(self, cfg, training):
        """Patch config to support training algorithm."""
        logger.info(f"Semi-SL task config!!!!: training={training}")
        if "task_adapt" in cfg:
            self.task_adapt_type = cfg["task_adapt"].get("type", None)
            self.task_adapt_op = cfg["task_adapt"].get("op", "REPLACE")
            self.configure_classes(cfg)

            if self.data_classes != self.model_classes:
                self.configure_task_data_pipeline(cfg)
            # TODO[JAEGUK]: configure_anchor is not working
            if cfg["task_adapt"].get("use_mpa_anchor", False):
                self.configure_anchor(cfg)
            if self.task_adapt_type == "mpa":
                self.configure_bbox_head(cfg)
            else:
                src_data_cfg = self.get_data_cfg(cfg, "train")
                src_data_cfg.pop("old_new_indices", None)

    @staticmethod
    def configure_unlabeled_dataloader(cfg: Config):
        """Patch for unlabled dataloader."""

        model_task = {"classification": "mmcls", "detection": "mmdet", "segmentation": "mmseg"}
        if "unlabeled" in cfg.data:
            task_lib_module = importlib.import_module(f"{model_task[cfg.model_task]}.datasets")
            dataset_builder = getattr(task_lib_module, "build_dataset")
            dataloader_builder = getattr(task_lib_module, "build_dataloader")

            dataset = build_dataset(cfg, "unlabeled", dataset_builder, consume=True)
            unlabeled_dataloader = build_dataloader(
                dataset,
                cfg,
                "unlabeled",
                dataloader_builder,
                distributed=cfg.distributed,
                consume=True,
            )

            custom_hooks = cfg.get("custom_hooks", [])
            updated = False
            for custom_hook in custom_hooks:
                if custom_hook["type"] == "ComposedDataLoadersHook":
                    custom_hook["data_loaders"] = [*custom_hook["data_loaders"], unlabeled_dataloader]
                    updated = True
            if not updated:
                custom_hooks.append(
                    ConfigDict(
                        type="ComposedDataLoadersHook",
                        data_loaders=unlabeled_dataloader,
                    )
                )
            cfg.custom_hooks = custom_hooks



"""Tests for Segmentation with OTX CLI"""
# Copyright (C) 2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#
import copy
import json
import os
from pathlib import Path
from timeit import default_timer as timer

import pytest

from otx.cli.registry import Registry
from tests.regression.regression_test_helpers import (
    REGRESSION_TEST_EPOCHS,
    TIME_LOG,
    get_result_dict,
    get_template_performance,
    load_regression_configuration,
)
from tests.test_suite.e2e_test_system import e2e_pytest_component
from tests.test_suite.run_test_command import (
    get_template_dir,
    nncf_optimize_testing,
    otx_deploy_openvino_testing,
    otx_export_testing,
    otx_train_testing,
    pot_optimize_testing,
)

from tests.regression.regression_command import (
    regression_eval_testing,
    regression_openvino_testing,
    regression_deployment_testing,
    regression_nncf_eval_testing,
    regression_pot_eval_testing,
    regression_train_time_testing,
    regression_eval_time_testing,
)

# Configurations for regression test.
TASK_TYPE = "segmentation"
TRAIN_TYPE = "supervised"
LABEL_TYPE = "multi_class"

otx_dir = os.getcwd()
templates = Registry(f"otx/algorithms/{TASK_TYPE}").filter(task_type=TASK_TYPE.upper()).templates
templates_ids = [template.model_template_id for template in templates]

result_dict = get_result_dict(TASK_TYPE)
result_dir = f"/tmp/regression_test_results/{TASK_TYPE}"
Path(result_dir).mkdir(parents=True, exist_ok=True)

segmentation_regression_config = load_regression_configuration(otx_dir, TASK_TYPE, TRAIN_TYPE, LABEL_TYPE)
segmentation_data_args = segmentation_regression_config["data_path"]
segmentation_data_args["train_params"] = ["params", "--learning_parameters.num_iters", REGRESSION_TEST_EPOCHS]


class TestRegressionSegmentation:
    def setup_method(self):
        self.label_type = LABEL_TYPE
        self.performance = {}

    def teardown_method(self):
        with open(f"{result_dir}/result.json", "w") as result_file:
            json.dump(result_dict, result_file, indent=4)

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        train_start_time = timer()
        otx_train_testing(template, tmp_dir_path, otx_dir, segmentation_data_args)
        train_elapsed_time = timer() - train_start_time

        infer_start_time = timer()
        test_result = regression_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            segmentation_data_args,
            segmentation_regression_config["regression_criteria"]["train"],
            self.performance[template.name],
        )
        infer_elapsed_time = timer() - infer_start_time

        self.performance[template.name][TIME_LOG["train_time"]] = round(train_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["infer_time"]] = round(infer_elapsed_time, 3)
        result_dict[TASK_TYPE][LABEL_TYPE][TRAIN_TYPE]["train"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train_kpi_test(self, template):
        results = result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["train"]
        performance = get_template_performance(results, template)

        kpi_train_result = regression_train_time_testing(
            train_time_criteria=segmentation_regression_config["kpi_e2e_train_time_criteria"]["train"],
            e2e_train_time=performance[template.name][TIME_LOG["train_time"]],
            template=template,
        )

        kpi_eval_result = regression_eval_time_testing(
            eval_time_criteria=segmentation_regression_config["kpi_e2e_eval_time_criteria"]["train"],
            e2e_eval_time=performance[template.name][TIME_LOG["infer_time"]],
            template=template,
        )

        assert kpi_train_result["passed"] is True, kpi_train_result["log"]
        assert kpi_eval_result["passed"] is True, kpi_eval_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train_cls_incr(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        sl_template_work_dir = get_template_dir(template, tmp_dir_path / TASK_TYPE)

        tmp_dir_path = tmp_dir_path / "seg_incr"
        config_cls_incr = load_regression_configuration(otx_dir, TASK_TYPE, "class_incr", self.label_type)
        args_cls_incr = config_cls_incr["data_path"]
        args_cls_incr[
            "--load-weights"
        ] = f"{sl_template_work_dir}/trained_{template.model_template_id}/models/weights.pth"
        args_cls_incr["train_params"] = ["params", "--learning_parameters.num_iters", REGRESSION_TEST_EPOCHS]

        train_start_time = timer()
        otx_train_testing(template, tmp_dir_path, otx_dir, args_cls_incr)
        train_elapsed_time = timer() - train_start_time

        infer_start_time = timer()
        test_result = regression_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            args_cls_incr,
            config_cls_incr["regression_criteria"]["train"],
            self.performance[template.name],
        )
        infer_elapsed_time = timer() - infer_start_time

        self.performance[template.name][TIME_LOG["train_time"]] = round(train_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["infer_time"]] = round(infer_elapsed_time, 3)
        result_dict[TASK_TYPE][self.label_type]["class_incr"]["train"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train_cls_incr_kpi_test(self, template):
        config_cls_incr = load_regression_configuration(otx_dir, TASK_TYPE, "class_incr", self.label_type)
        results = result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["train"]
        performance = get_template_performance(results, template)

        kpi_train_result = regression_train_time_testing(
            train_time_criteria=config_cls_incr["kpi_e2e_train_time_criteria"]["train"],
            e2e_train_time=performance[template.name][TIME_LOG["train_time"]],
            template=template,
        )

        kpi_eval_result = regression_eval_time_testing(
            eval_time_criteria=config_cls_incr["kpi_e2e_eval_time_criteria"]["train"],
            e2e_eval_time=performance[template.name][TIME_LOG["infer_time"]],
            template=template,
        )

        assert kpi_train_result["passed"] is True, kpi_train_result["log"]
        assert kpi_eval_result["passed"] is True, kpi_eval_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train_semisl(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        tmp_dir_path = tmp_dir_path / f"{TASK_TYPE}/test_semisl"
        config_semisl = load_regression_configuration(otx_dir, TASK_TYPE, "semi_supervised", LABEL_TYPE)
        args_semisl = config_semisl["data_path"]

        args_semisl["train_params"] = [
            "params",
            "--learning_parameters.num_iters",
            REGRESSION_TEST_EPOCHS,
            "--algo_backend.train_type",
            "Semisupervised",
        ]
        train_start_time = timer()
        otx_train_testing(template, tmp_dir_path, otx_dir, args_semisl)
        train_elapsed_time = timer() - train_start_time

        args_semisl.pop("train_params")
        infer_start_time = timer()
        test_result = regression_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            args_semisl,
            config_semisl["regression_criteria"]["train"],
            self.performance[template.name],
        )
        infer_elapsed_time = timer() - infer_start_time

        self.performance[template.name][TIME_LOG["train_time"]] = round(train_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["infer_time"]] = round(infer_elapsed_time, 3)
        result_dict[TASK_TYPE][LABEL_TYPE]["semi_supervised"]["train"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train_semisl_kpi_test(self, template):
        config_semisl = load_regression_configuration(otx_dir, TASK_TYPE, "semi_supervised", LABEL_TYPE)
        results = result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["train"]
        performance = get_template_performance(results, template)

        kpi_train_result = regression_train_time_testing(
            train_time_criteria=config_semisl["kpi_e2e_train_time_criteria"]["train"],
            e2e_train_time=performance[template.name][TIME_LOG["train_time"]],
            template=template,
        )

        kpi_eval_result = regression_eval_time_testing(
            eval_time_criteria=config_semisl["kpi_e2e_eval_time_criteria"]["train"],
            e2e_eval_time=performance[template.name][TIME_LOG["infer_time"]],
            template=template,
        )

        assert kpi_train_result["passed"] is True, kpi_train_result["log"]
        assert kpi_eval_result["passed"] is True, kpi_eval_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train_selfsl(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        tmp_dir_path = tmp_dir_path / f"{TASK_TYPE}/test_selfsl"
        config_selfsl = load_regression_configuration(otx_dir, TASK_TYPE, "self_supervised", LABEL_TYPE)
        args_selfsl = config_selfsl["data_path"]

        selfsl_train_args = copy.deepcopy(args_selfsl)
        selfsl_train_args["train_params"] = ["params", "--algo_backend.train_type", "Selfsupervised"]

        # Self-supervised Training
        train_start_time = timer()
        otx_train_testing(template, tmp_dir_path, otx_dir, selfsl_train_args)
        train_elapsed_time = timer() - train_start_time

        # Supervised Training
        template_work_dir = get_template_dir(template, tmp_dir_path)
        new_tmp_dir_path = tmp_dir_path / "test_supervised"
        args_selfsl["train_params"] = ["params", "--learning_parameters.num_iters", REGRESSION_TEST_EPOCHS]
        args_selfsl["--val-data-roots"] = segmentation_data_args["--val-data-roots"]
        args_selfsl["--test-data-roots"] = segmentation_data_args["--test-data-roots"]
        args_selfsl["--load-weights"] = f"{template_work_dir}/trained_{template.model_template_id}/models/weights.pth"
        otx_train_testing(template, new_tmp_dir_path, otx_dir, args_selfsl)

        # Evaluation with self + supervised training model
        args_selfsl.pop("--load-weights")
        infer_start_time = timer()
        test_result = regression_eval_testing(
            template,
            new_tmp_dir_path,
            otx_dir,
            args_selfsl,
            config_selfsl["regression_criteria"]["train"],
            self.performance[template.name],
        )
        infer_elapsed_time = timer() - infer_start_time

        self.performance[template.name][TIME_LOG["train_time"]] = round(train_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["infer_time"]] = round(infer_elapsed_time, 3)
        result_dict[TASK_TYPE][self.label_type]["self_supervised"]["train"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_train_selfsl_kpi_test(self, template):
        config_selfsl = load_regression_configuration(otx_dir, TASK_TYPE, "self_supervised", LABEL_TYPE)
        results = result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["train"]
        performance = get_template_performance(results, template)

        kpi_train_result = regression_train_time_testing(
            train_time_criteria=config_selfsl["kpi_e2e_train_time_criteria"]["train"],
            e2e_train_time=performance[template.name][TIME_LOG["train_time"]],
            template=template,
        )

        kpi_eval_result = regression_eval_time_testing(
            eval_time_criteria=config_selfsl["kpi_e2e_eval_time_criteria"]["train"],
            e2e_eval_time=performance[template.name][TIME_LOG["infer_time"]],
            template=template,
        )

        assert kpi_train_result["passed"] is True, kpi_train_result["log"]
        assert kpi_eval_result["passed"] is True, kpi_eval_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_export_eval_openvino(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        export_start_time = timer()
        otx_export_testing(template, tmp_dir_path)
        export_elapsed_time = timer() - export_start_time

        export_eval_start_time = timer()
        test_result = regression_openvino_testing(
            template,
            tmp_dir_path,
            otx_dir,
            segmentation_data_args,
            threshold=0.05,
            criteria=segmentation_regression_config["regression_criteria"]["export"],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        export_eval_elapsed_time = timer() - export_eval_start_time

        self.performance[template.name][TIME_LOG["export_time"]] = round(export_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["export_eval_time"]] = round(export_eval_elapsed_time, 3)
        result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["export"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_otx_deploy_eval_deployment(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        deploy_start_time = timer()
        otx_deploy_openvino_testing(template, tmp_dir_path, otx_dir, segmentation_data_args)
        deploy_elapsed_time = timer() - deploy_start_time

        deploy_eval_start_time = timer()
        test_result = regression_deployment_testing(
            template,
            tmp_dir_path,
            otx_dir,
            segmentation_data_args,
            threshold=0.0,
            criteria=segmentation_regression_config["regression_criteria"]["deploy"],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        deploy_eval_elapsed_time = timer() - deploy_eval_start_time

        self.performance[template.name][TIME_LOG["deploy_time"]] = round(deploy_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["deploy_eval_time"]] = round(deploy_eval_elapsed_time, 3)
        result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["deploy"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_nncf_optimize_eval(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        if template.entrypoints.nncf is None:
            pytest.skip("nncf entrypoint is none")

        nncf_start_time = timer()
        nncf_optimize_testing(template, tmp_dir_path, otx_dir, segmentation_data_args)
        nncf_elapsed_time = timer() - nncf_start_time

        nncf_eval_start_time = timer()
        test_result = regression_nncf_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            segmentation_data_args,
            threshold=0.01,
            criteria=segmentation_regression_config["regression_criteria"]["nncf"],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        nncf_eval_elapsed_time = timer() - nncf_eval_start_time

        self.performance[template.name][TIME_LOG["nncf_time"]] = round(nncf_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["nncf_eval_time"]] = round(nncf_eval_elapsed_time, 3)
        result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["nncf"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]

    @e2e_pytest_component
    @pytest.mark.parametrize("template", templates, ids=templates_ids)
    def test_pot_optimize_eval(self, template, tmp_dir_path):
        self.performance[template.name] = {}

        tmp_dir_path = tmp_dir_path / TASK_TYPE
        pot_start_time = timer()
        pot_optimize_testing(template, tmp_dir_path, otx_dir, segmentation_data_args)
        pot_elapsed_time = timer() - pot_start_time

        pot_eval_start_time = timer()
        test_result = regression_pot_eval_testing(
            template,
            tmp_dir_path,
            otx_dir,
            segmentation_data_args,
            criteria=segmentation_regression_config["regression_criteria"]["pot"],
            reg_threshold=0.10,
            result_dict=self.performance[template.name],
        )
        pot_eval_elapsed_time = timer() - pot_eval_start_time

        self.performance[template.name][TIME_LOG["pot_time"]] = round(pot_elapsed_time, 3)
        self.performance[template.name][TIME_LOG["pot_eval_time"]] = round(pot_eval_elapsed_time, 3)
        result_dict[TASK_TYPE][self.label_type][TRAIN_TYPE]["pot"].append(self.performance)

        assert test_result["passed"] is True, test_result["log"]



import pytest

from otx.algorithms.common.adapters.mmcv.utils import automatic_bs
from otx.algorithms.common.adapters.mmcv.utils import adapt_batch_size
from otx.algorithms.common.adapters.mmcv.utils.automatic_bs import SubDataset

DEFAULT_BS = 8
DEFAULT_LR = 0.001
TRAINSET_SIZE = 100


def bs_adapt_func(train_func, current_bs, trainset_size):
    train_func(current_bs)
    train_func(current_bs // 2)
    return current_bs // 2


@pytest.fixture
def mock_adapt_func(mocker):
    mock_func = mocker.patch.object(automatic_bs, "adapt_torch_model_bs")
    mock_func.side_effect = bs_adapt_func
    return mock_func


@pytest.fixture
def common_cfg(mocker):
    mock_cfg = mocker.MagicMock()
    mock_cfg.runner = {"type": "EpochRunnerWithCancel", "max_epochs": 100}
    mock_cfg.custom_hooks = [{"type": "AdaptiveTrainSchedulingHook", "enable_eval_before_run": True}]
    mock_cfg.optimizer.lr = DEFAULT_LR
    return mock_cfg


@pytest.fixture
def mock_cfg_not_action(common_cfg):
    common_cfg.data.train_dataloader = {"samples_per_gpu": DEFAULT_BS}
    return common_cfg


@pytest.fixture
def mock_cfg_action(common_cfg):
    common_cfg.data.videos_per_gpu = DEFAULT_BS
    common_cfg.domain = "ACTION_CLASSIFICATION"
    return common_cfg


@pytest.fixture
def mock_dataset(mocker):
    mock_ds = [mocker.MagicMock()]
    mock_ds[0].__len__.return_value = TRAINSET_SIZE
    return mock_ds


def test_adapt_batch_size_not_action_task(mocker, mock_adapt_func, mock_cfg_not_action, mock_dataset):
    # prepare
    mock_train_func = mocker.MagicMock()

    # execute
    adapt_batch_size(mock_train_func, mock_cfg_not_action, mock_dataset, False)

    # check adapted batch size is applied
    assert mock_cfg_not_action.data.train_dataloader["samples_per_gpu"] == DEFAULT_BS // 2
    # check leanring rate is updated depending on adapted batch size
    assert mock_cfg_not_action.optimizer.lr == pytest.approx(DEFAULT_LR / 2)
    # check adapt function gets proper arguments
    assert mock_adapt_func.call_args.kwargs["current_bs"] == DEFAULT_BS
    assert mock_adapt_func.call_args.kwargs["trainset_size"] == TRAINSET_SIZE
    # check length of dataset is decreased to reduce time
    assert len(mock_train_func.call_args_list[0].kwargs["dataset"][0]) == DEFAULT_BS
    assert len(mock_train_func.call_args_list[1].kwargs["dataset"][0]) == DEFAULT_BS // 2
    # check max epoch is set as 1 to reduce time
    assert mock_train_func.call_args_list[0].kwargs["cfg"].runner["max_epochs"] == 1
    assert mock_train_func.call_args_list[1].kwargs["cfg"].runner["max_epochs"] == 1
    # check eval before run is disabled to reduce time
    assert not mock_train_func.call_args_list[0].kwargs["cfg"].custom_hooks[0]["enable_eval_before_run"]
    assert not mock_train_func.call_args_list[1].kwargs["cfg"].custom_hooks[0]["enable_eval_before_run"]


def test_adapt_batch_size_action_task(mocker, mock_adapt_func, mock_cfg_action, mock_dataset):
    # prepare
    mock_train_func = mocker.MagicMock()

    # execute
    adapt_batch_size(mock_train_func, mock_cfg_action, mock_dataset, True)

    # check adapted batch size is applied
    assert mock_cfg_action.data.videos_per_gpu == DEFAULT_BS // 2
    # check leanring rate is updated depending on adapted batch size
    assert mock_cfg_action.optimizer.lr == pytest.approx(DEFAULT_LR / 2)
    # check adapt function gets proper arguments
    assert mock_adapt_func.call_args.kwargs["current_bs"] == DEFAULT_BS
    assert mock_adapt_func.call_args.kwargs["trainset_size"] == TRAINSET_SIZE
    # check length of dataset is decreased to reduce time
    assert len(mock_train_func.call_args_list[0].kwargs["dataset"][0]) == DEFAULT_BS
    assert len(mock_train_func.call_args_list[1].kwargs["dataset"][0]) == DEFAULT_BS // 2
    # check max epoch is set as 1 to reduce time
    assert mock_train_func.call_args_list[0].kwargs["cfg"].runner["max_epochs"] == 1
    assert mock_train_func.call_args_list[1].kwargs["cfg"].runner["max_epochs"] == 1
    # check eval before run is enabled if validate is set as True
    assert mock_train_func.call_args_list[0].kwargs["cfg"].custom_hooks[0]["enable_eval_before_run"]
    assert mock_train_func.call_args_list[1].kwargs["cfg"].custom_hooks[0]["enable_eval_before_run"]


class TestSubDataset:
    @pytest.fixture(autouse=True)
    def set_up(self, mocker):
        self.num_samples = 3
        self.fullset = mocker.MagicMock()
        self.sub_dataset = SubDataset(self.fullset, self.num_samples)

    def test_init(self, mocker):
        fullset = mocker.MagicMock()
        SubDataset(fullset, 3)

    @pytest.mark.parametrize("num_sampels", [-1, 0])
    def test_init_w_wrong_num_sampels(self, mocker, num_sampels):
        fullset = mocker.MagicMock()
        with pytest.raises(ValueError):
            SubDataset(fullset, num_sampels)

    def test_len(self):
        assert len(self.sub_dataset) == self.num_samples

    def test_getitem(self):
        self.sub_dataset[0]
        self.fullset.__getitem__.assert_called_once_with(0)

    def test_getattr(self):
        self.fullset.data = "data"
        assert self.sub_dataset.data == "data"

    def test_flag(self):
        assert len(self.sub_dataset.flag) == self.num_samples



# Copyright (C) 2021-2023 Intel Corporation
# SPDX-Lggggcense-Identifier: Apache-2.0
#

import pytest

from otx.core.ov.ops.sorting_maximization import (
    NonMaxSuppressionV5,
    NonMaxSuppressionV9,
    TopKV3,
)
from tests.test_suite.e2e_test_system import e2e_pytest_unit


class TestTopKV3:
    @e2e_pytest_unit
    def test_invalid_attr(self):
        with pytest.raises(ValueError):
            TopKV3("dummy", shape=(1,), axis=1, mode="error", sort="value", index_element_type="i32")

        with pytest.raises(ValueError):
            TopKV3("dummy", shape=(1,), axis=1, mode="max", sort="error", index_element_type="i32")

        with pytest.raises(ValueError):
            TopKV3("dummy", shape=(1,), axis=1, mode="max", sort="value", index_element_type="f32")

    @e2e_pytest_unit
    def test_forward(self):
        op = TopKV3("dummy", shape=(1,), axis=1, mode="max", sort="value")
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy")


class TestNonMaxSuppressionV5:
    @e2e_pytest_unit
    def test_forward(self):
        op = NonMaxSuppressionV5("dummy", shape=(1,))
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy", "dummy")


class TestNonMaxSuppressionV9:
    @e2e_pytest_unit
    def test_forward(self):
        op = NonMaxSuppressionV9("dummy", shape=(1,))
        with pytest.raises(NotImplementedError):
            op("dummy", "dummy", "dummy")



"""Module for defining semi-supervised classification head for multi-label classification task."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import torch
from mmcls.models.builder import HEADS, build_loss
from torch import nn

from otx.algorithms.classification.adapters.mmcls.models.heads.custom_multi_label_linear_cls_head import (
    CustomMultiLabelLinearClsHead,
)
from otx.algorithms.classification.adapters.mmcls.models.heads.custom_multi_label_non_linear_cls_head import (
    CustomMultiLabelNonLinearClsHead,
)


def generate_aux_mlp(aux_mlp_cfg: dict, in_channels: int):
    """Generate auxiliary MLP."""
    out_channels = aux_mlp_cfg["out_channels"]
    if out_channels <= 0:
        raise ValueError(f"out_channels={out_channels} must be a positive integer")
    if "hid_channels" in aux_mlp_cfg and aux_mlp_cfg["hid_channels"] > 0:
        hid_channels = aux_mlp_cfg["hid_channels"]
        mlp = nn.Sequential(
            nn.Linear(in_features=in_channels, out_features=hid_channels),
            nn.ReLU(inplace=True),
            nn.Linear(in_features=hid_channels, out_features=out_channels),
        )
    else:
        mlp = nn.Linear(in_features=in_channels, out_features=out_channels)

    return mlp


class EMAMeter:
    """Exponential Moving Average Meter class."""

    def __init__(self, alpha=0.9):
        """Initialize the Exponential Moving Average Meter.

        Args:
            alpha (float): Smoothing factor for the exponential moving average. Defaults to 0.9.
        """
        self.alpha = alpha
        self.val = 0

    def reset(self):
        """Reset the Exponential Moving Average Meter."""
        self.val = 0

    def update(self, val):
        """Update the Exponential Moving Average Meter with new value.

        Args:
            val (float): New value to update the meter.
        """
        self.val = self.alpha * self.val + (1 - self.alpha) * val


class LossBalancer:
    """Loss Balancer class."""

    def __init__(self, num_losses, weights=None, ema_weight=0.7) -> None:
        """Initialize the Loss Balancer.

        Args:
            num_losses (int): Number of losses to balance.
            weights (list): List of weights to be applied to each loss. If None, equal weights are applied.
            ema_weight (float): Smoothing factor for the exponential moving average meter. Defaults to 0.7.
        """
        self.epsilon = 1e-9
        self.avg_estimators = [EMAMeter(ema_weight) for _ in range(num_losses)]

        if weights is not None:
            assert len(weights) == num_losses
            self.final_weights = weights
        else:
            self.final_weights = [1.0] * num_losses

    def balance_losses(self, losses):
        """Balance the given losses using the weights and exponential moving average.

        Args:
            losses (list): List of losses to be balanced.

        Returns:
            total_loss (float): Balanced loss value.
        """
        total_loss = 0.0
        for i, loss in enumerate(losses):
            self.avg_estimators[i].update(float(loss))
            total_loss += (
                self.final_weights[i] * loss / (self.avg_estimators[i].val + self.epsilon) * self.avg_estimators[0].val
            )
        return total_loss


class SemiMultilabelClsHead:
    """Multilabel Classification head for Semi-SL.

    Args:
        unlabeled_coef (float): unlabeled loss coefficient, default is 1.0.
        use_dynamic_loss_weighting (boolean): whether to use dynamic unlabeled loss weighting, default is True.
        aux_loss (dict, optional): auxiliary loss function, default is None.
    """

    def __init__(
        self,
        unlabeled_coef=0.1,
        use_dynamic_loss_weighting=True,
        aux_loss=None,
    ):
        aux_loss = (
            aux_loss if aux_loss else dict(type="BarlowTwinsLoss", off_diag_penality=1.0 / 128.0, loss_weight=1.0)
        )
        self.unlabeled_coef = unlabeled_coef
        self.use_dynamic_loss_weighting = use_dynamic_loss_weighting
        self.aux_loss = build_loss(aux_loss)
        if self.use_dynamic_loss_weighting:
            self.loss_balancer = LossBalancer(2, [1.0, unlabeled_coef])
        else:
            self.loss_balancer = None
        self.num_pseudo_label = 0

    def loss(self, logits, gt_label, features):
        """Loss function in which unlabeled data is considered.

        Args:
            logits (Tensor): Labeled data logits
            gt_label (Tensor): target features for labeled data
            features (set): (weak data features, strong data features)

        Returns:
            dict[str, Tensor]: A dictionary of loss components.
        """
        num_samples = gt_label.shape[0]
        # map difficult examples to positive ones
        _gt_label = torch.abs(gt_label)
        l_labeled = self.compute_loss(
            logits,
            _gt_label,
            avg_factor=num_samples,
        )

        features_weak, features_strong = features
        aux_loss = self.aux_loss(features_weak, features_strong)

        losses = dict(loss=0.0)
        if self.use_dynamic_loss_weighting:
            losses["loss"] = self.loss_balancer.balance_losses((l_labeled, aux_loss))
        else:
            losses["loss"] = l_labeled + self.unlabeled_coef * aux_loss
        losses["unlabeled_loss"] = self.unlabeled_coef * aux_loss

        return losses

    def forward_train_with_last_layers(self, x, gt_label, final_cls_layer, final_emb_layer):
        """Forwards multilabel semi-sl head and losses.

        Args:
            x (dict): dict(labeled_weak. labeled_strong, unlabeled_weak, unlabeled_strong) or NxC input features.
            gt_label (Tensor): NxC target features.
            final_cls_layer (nn.Linear or nn.Sequential): a final layer forwards feature from backbone.
            final_emb_layer (nn.Linear or nn.Sequential): a final layer forwards embeddings from backbone.

        Returns:
            dict[str, Tensor]: A dictionary of loss components.
        """
        logits = final_cls_layer(x["labeled_weak"])
        features_weak = torch.cat((final_emb_layer(x["labeled_weak"]), final_emb_layer(x["unlabeled_weak"])))
        features_strong = torch.cat((final_emb_layer(x["labeled_strong"]), final_emb_layer(x["unlabeled_strong"])))
        features = (features_weak, features_strong)
        losses = self.loss(logits, gt_label, features)
        return losses


@HEADS.register_module()
class SemiLinearMultilabelClsHead(SemiMultilabelClsHead, CustomMultiLabelLinearClsHead):
    """Linear multilabel classification head for Semi-SL.

    Args:
        num_classes (int): The number of classes of dataset used for training
        in_channels (int): The channels of input data from classifier
        scale (float): Scale for metric learning loss
        normalized (boolean): flag that enables metric learining in loss,
        aux_mlp (dict): Config for embeddings MLP
        loss (dict): configuration of loss, default is CrossEntropyLoss
        unlabeled_coef (float): unlabeled loss coefficient, default is 1.0
        use_dynamic_loss_weighting (boolean): whether to use dynamic unlabeled loss weighting, default is True
    """

    def __init__(
        self,
        num_classes,
        in_channels,
        scale=1.0,
        normalized=False,
        aux_mlp=None,
        loss=None,
        unlabeled_coef=0.1,
        aux_loss=None,
        use_dynamic_loss_weighting=True,
    ):  # pylint: disable=too-many-arguments
        if in_channels <= 0:
            raise ValueError(f"in_channels={in_channels} must be a positive integer")
        if num_classes <= 0:
            raise ValueError("at least one class must be exist num_classes.")
        aux_mlp = aux_mlp if aux_mlp else dict(hid_channels=0, out_channels=1024)
        loss = loss if loss else dict(type="CrossEntropyLoss", loss_weight=1.0)
        aux_loss = (
            aux_loss if aux_loss else dict(type="BarlowTwinsLoss", off_diag_penality=1.0 / 128.0, loss_weight=1.0)
        )
        CustomMultiLabelLinearClsHead.__init__(self, num_classes, in_channels, normalized, scale, loss)
        SemiMultilabelClsHead.__init__(self, unlabeled_coef, use_dynamic_loss_weighting, aux_loss)

        self.aux_mlp = generate_aux_mlp(aux_mlp, in_channels)

    def loss(self, logits, gt_label, features):
        """Calculate loss for given logits/gt_label."""
        return SemiMultilabelClsHead.loss(self, logits, gt_label, features)

    def forward(self, x):
        """Forward fuction of SemiLinearMultilabelClsHead class."""
        return self.simple_test(x)

    def forward_train(self, cls_score, gt_label):
        """Forward_train fuction of SemiLinearMultilabelClsHead class."""
        return self.forward_train_with_last_layers(
            cls_score, gt_label, final_cls_layer=self.fc, final_emb_layer=self.aux_mlp
        )


@HEADS.register_module()
class SemiNonLinearMultilabelClsHead(SemiMultilabelClsHead, CustomMultiLabelNonLinearClsHead):
    """Non-linear classification head for Semi-SL.

    Args:
        num_classes (int): The number of classes of dataset used for training
        in_channels (int): The channels of input data from classifier
        hid_channels (int): Number of channels of hidden layer.
        scale (float): Scale for metric learning loss
        normalized (boolean): flag that enables metric learining in loss,
        aux_mlp (dict): Config for embeddings MLP
        act_cfg (dict): Config of activation layer
        loss (dict): configuration of loss, default is CrossEntropyLoss
        topk (set): evaluation topk score, default is (1, )
        unlabeled_coef (float): unlabeled loss coefficient, default is 0.1
        use_dynamic_loss_weighting (boolean): whether to use dynamic unlabeled loss weighting, default is True
    """

    def __init__(
        self,
        num_classes,
        in_channels,
        hid_channels=1280,
        scale=1.0,
        normalized=False,
        aux_mlp=None,
        act_cfg=None,
        loss=None,
        aux_loss=None,
        dropout=False,
        unlabeled_coef=0.1,
        use_dynamic_loss_weighting=True,
    ):  # pylint: disable=too-many-arguments
        if in_channels <= 0:
            raise ValueError(f"in_channels={in_channels} must be a positive integer")
        if num_classes <= 0:
            raise ValueError("at least one class must be exist num_classes.")
        aux_mlp = aux_mlp if aux_mlp else dict(hid_channels=0, out_channels=1024)
        act_cfg = act_cfg if act_cfg else dict(type="ReLU")
        loss = loss if loss else dict(type="CrossEntropyLoss", loss_weight=1.0)
        aux_loss = (
            aux_loss if aux_loss else dict(type="BarlowTwinsLoss", off_diag_penality=1.0 / 128.0, loss_weight=1.0)
        )
        CustomMultiLabelNonLinearClsHead.__init__(
            self,
            num_classes,
            in_channels,
            hid_channels=hid_channels,
            act_cfg=act_cfg,
            loss=loss,
            dropout=dropout,
            scale=scale,
            normalized=normalized,
        )
        SemiMultilabelClsHead.__init__(self, unlabeled_coef, use_dynamic_loss_weighting, aux_loss)

        self.aux_mlp = generate_aux_mlp(aux_mlp, in_channels)

    def loss(self, logits, gt_label, features):
        """Calculate loss for given logits/gt_label."""
        return SemiMultilabelClsHead.loss(self, logits, gt_label, features)

    def forward(self, x):
        """Forward fuction of SemiNonLinearMultilabelClsHead class."""
        return self.simple_test(x)

    def forward_train(self, cls_score, gt_label):
        """Forward_train fuction of SemiNonLinearMultilabelClsHead class."""
        return self.forward_train_with_last_layers(
            cls_score, gt_label, final_cls_layer=self.classifier, final_emb_layer=self.aux_mlp
        )



"""Custom FCN head."""

# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

from typing import Dict, List, Optional

import torch
from mmcv.runner import force_fp32
from mmseg.models.builder import HEADS
from mmseg.models.decode_heads.fcn_head import FCNHead
from mmseg.models.losses import accuracy
from mmseg.ops import resize
from torch import nn

from otx.algorithms.segmentation.adapters.mmseg.models.utils import IterativeAggregator
from otx.algorithms.segmentation.adapters.mmseg.utils import (
    get_valid_label_mask_per_batch,
)


@HEADS.register_module()
class CustomFCNHead(FCNHead):
    """Custom Fully Convolution Networks for Semantic Segmentation.

    This FCN Head added head aggregator used in Lite-HRNet by
    DepthwiseSeparableConvModule.
    Please refer to https://github.com/HRNet/Lite-HRNet.

    Args:
        enable_aggregator (bool): If true, will use Lite-HRNet aggregator
            concating all inputs from backbone by DepthwiseSeparableConvModule.
        aggregator_min_channels (int, optional): The number of channels of output of aggregator.
            It would work only if enable_aggregator is true.
        aggregator_merge_norm (str, optional): normalize the output of expanders of aggregator.
            options : "none", "channel", or None
        aggregator_use_concat (str, optional): Whether to concat the last input
            with the output of expanders.
    """

    def __init__(
        self,
        enable_aggregator: bool = False,
        aggregator_min_channels: Optional[int] = None,
        aggregator_merge_norm: Optional[str] = None,
        aggregator_use_concat: bool = False,
        *args,
        **kwargs
    ):

        in_channels = kwargs.get("in_channels")
        in_index = kwargs.get("in_index")
        norm_cfg = kwargs.get("norm_cfg")
        conv_cfg = kwargs.get("conv_cfg")
        input_transform = kwargs.get("input_transform")

        aggregator = None
        if enable_aggregator:  # Lite-HRNet aggregator
            assert isinstance(in_channels, (tuple, list))
            assert len(in_channels) > 1

            aggregator = IterativeAggregator(
                in_channels=in_channels,
                min_channels=aggregator_min_channels,
                conv_cfg=conv_cfg,
                norm_cfg=norm_cfg,
                merge_norm=aggregator_merge_norm,
                use_concat=aggregator_use_concat,
            )

            aggregator_min_channels = aggregator_min_channels if aggregator_min_channels is not None else 0
            # change arguments temporarily
            kwargs["in_channels"] = max(in_channels[0], aggregator_min_channels)
            kwargs["input_transform"] = None
            if in_index is not None:
                kwargs["in_index"] = in_index[0]

        super().__init__(*args, **kwargs)

        self.aggregator = aggregator

        # re-define variables
        self.in_channels = in_channels
        self.input_transform = input_transform
        self.in_index = in_index

        self.ignore_index = 255

        # get rid of last activation of convs module
        if self.act_cfg:
            self.convs[-1].with_activation = False
            delattr(self.convs[-1], "activate")

        if kwargs.get("init_cfg", {}):
            self.init_weights()

    def _transform_inputs(self, inputs: torch.Tensor):
        if self.aggregator is not None:
            inputs = self.aggregator(inputs)[0]
        else:
            inputs = super()._transform_inputs(inputs)

        return inputs

    def forward_train(
        self, inputs: torch.Tensor, img_metas: List[Dict], gt_semantic_seg: torch.Tensor, train_cfg: Dict
    ):
        """Forward function for training.

        Args:
            inputs (list[Tensor]): List of multi-level img features.
            img_metas (list[dict]): List of image info dict where each dict
                has: 'img_shape', 'scale_factor', 'flip', and may also contain
                'filename', 'ori_shape', 'pad_shape', and 'img_norm_cfg'.
                For details on the values of these keys see
                `mmseg/datasets/pipelines/formatting.py:Collect`.
            gt_semantic_seg (Tensor): Semantic segmentation masks
                used if the architecture supports semantic segmentation task.
            train_cfg (dict): The training config.

        Returns:
            dict[str, Tensor]: a dictionary of loss components
        """
        seg_logits = self(inputs)
        valid_label_mask = get_valid_label_mask_per_batch(img_metas, self.num_classes)
        losses = self.losses(seg_logits, gt_semantic_seg, valid_label_mask=valid_label_mask)
        return losses

    @force_fp32(apply_to=("seg_logit",))
    def losses(self, seg_logit: torch.Tensor, seg_label: torch.Tensor, valid_label_mask: Optional[torch.Tensor] = None):
        """Compute segmentation loss."""
        loss = dict()

        seg_logit = resize(input=seg_logit, size=seg_label.shape[2:], mode="bilinear", align_corners=self.align_corners)
        if self.sampler is not None:
            seg_weight = self.sampler.sample(seg_logit, seg_label)
        else:
            seg_weight = None
        seg_label = seg_label.squeeze(1)

        if not isinstance(self.loss_decode, nn.ModuleList):
            losses_decode = [self.loss_decode]
        else:
            losses_decode = self.loss_decode
        for loss_decode in losses_decode:
            valid_label_mask_cfg = dict()
            if loss_decode.loss_name == "loss_ce_ignore":
                valid_label_mask_cfg["valid_label_mask"] = valid_label_mask
            if loss_decode.loss_name not in loss:
                loss[loss_decode.loss_name] = loss_decode(
                    seg_logit, seg_label, weight=seg_weight, ignore_index=self.ignore_index, **valid_label_mask_cfg
                )
            else:
                loss[loss_decode.loss_name] += loss_decode(
                    seg_logit, seg_label, weight=seg_weight, ignore_index=self.ignore_index, **valid_label_mask_cfg
                )

        loss["acc_seg"] = accuracy(seg_logit, seg_label, ignore_index=self.ignore_index)

        return loss



"""Backbone of pytorchcv for mmdetection backbones."""
# Copyright (C) 2023 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import os

from mmcv.cnn import build_activation_layer, build_norm_layer
from mmcv.runner import get_dist_info
from mmdet.models.builder import BACKBONES
from mmdet.utils.logger import get_root_logger
from pytorchcv.model_provider import _models
from pytorchcv.models.model_store import download_model
from torch import distributed, nn
from torch.nn.modules.batchnorm import _BatchNorm

# TODO: Need to fix pylint issues
# pylint: disable=protected-access, abstract-method, no-value-for-parameter, assignment-from-no-return


def replace_activation(model, activation_cfg):
    """Replace activate funtion."""
    for name, module in model._modules.items():
        if len(list(module.children())) > 0:
            model._modules[name] = replace_activation(module, activation_cfg)
        if name == "activ":
            if activation_cfg["type"] == "torch_swish":
                model._modules[name] = nn.SiLU()
            else:
                model._modules[name] = build_activation_layer(activation_cfg)
    return model


def replace_norm(model, cfg):
    """Replace norm funtion."""
    for name, module in model._modules.items():
        if len(list(module.children())) > 0:
            model._modules[name] = replace_norm(module, cfg)
        if name == "bn":
            model._modules[name] = build_norm_layer(cfg, num_features=module.num_features)[1]
    return model


def multioutput_forward(self, x):
    """Multioutput forward function for new model (copy from mmdet older)."""
    outputs = []
    y = x

    last_stage = max(self.out_indices)
    for i, stage in enumerate(self.features):
        y = stage(y)
        s_verbose = str(i) + " " + str(y.shape)
        if i in self.out_indices:
            outputs.append(y)
            s_verbose += "*"
        if self.verbose:
            print(s_verbose)
        if i == last_stage:
            break

    return outputs


def train(self, mode=True):
    """Train forward function for new model (copy from mmdet older)."""
    super(self.__class__, self).train(mode)

    for i in range(self.frozen_stages + 1):
        feature = self.features[i]
        feature.eval()
        for param in feature.parameters():
            param.requires_grad = False

    if mode and self.norm_eval:
        for module in self.modules():
            # trick: eval have effect on BatchNorm only
            if isinstance(module, _BatchNorm):
                module.eval()


def init_weights(self, pretrained=True):
    """Init weights function for new model (copy from mmdet)."""
    if pretrained:
        rank, world_size = get_dist_info()
        if rank == 0:
            # Make sure that model is fetched to the local storage.
            download_model(net=self, model_name=self.model_name, local_model_store_dir_path=self.models_cache_root)
            if world_size > 1:
                distributed.barrier()
        else:
            # Wait for model to be in the local storage, then load it.
            distributed.barrier()
            download_model(net=self, model_name=self.model_name, local_model_store_dir_path=self.models_cache_root)


def generate_backbones():
    """Generate backbones of pytorchcv funtion."""
    logger = get_root_logger()

    for model_name, model_getter in _models.items():

        def closure(model_name, model_getter):
            """Get Model builder for mmcv (copy from mmdet old version)."""

            class CustomModelGetter(nn.Module):
                """Custom Model getter class."""

                def __init__(
                    self,
                    *args,
                    out_indices=None,
                    frozen_stages=0,
                    norm_eval=False,
                    verbose=False,
                    activation_cfg=None,
                    norm_cfg=None,
                    **kwargs,
                ):
                    super().__init__()
                    models_cache_root = kwargs.get("root", os.path.join("~", ".torch", "models"))
                    is_pretrained = kwargs.get("pretrained", False)
                    logger.warning(
                        f"Init model {model_name}, pretrained={is_pretrained}, models cache {models_cache_root}"
                    )
                    model = model_getter(*args, **kwargs)
                    if activation_cfg:
                        model = replace_activation(model, activation_cfg)
                    if norm_cfg:
                        model = replace_norm(model, norm_cfg)
                    model.out_indices = out_indices
                    model.frozen_stages = frozen_stages
                    model.norm_eval = norm_eval
                    model.verbose = verbose
                    model.model_name = model_name
                    model.models_cache_root = models_cache_root
                    if hasattr(model, "features") and isinstance(model.features, nn.Sequential):
                        # Save original forward, just in case.
                        model.forward_single_output = model.forward
                        model.forward = multioutput_forward.__get__(model)
                        model.init_weights = init_weights.__get__(model)
                        model.train = train.__get__(model)

                        model.output = None
                        for i, _ in enumerate(model.features):
                            if i > max(out_indices):
                                model.features[i] = None
                    else:
                        raise ValueError(
                            "Failed to automatically wrap backbone network. "
                            f"Object of type {model.__class__} has no valid attribute called "
                            "'features'."
                        )
                    self.__dict__.update(model.__dict__)

            CustomModelGetter.__name__ = model_name
            return CustomModelGetter

        BACKBONES.register_module(name=model_name, module=closure(model_name, model_getter))


generate_backbones()



"""EfficientNet-V2 config for semi-supervised multi-class classification."""

# pylint: disable=invalid-name

_base_ = ["../../../../../recipes/stages/classification/semisl.yaml", "../../base/models/efficientnet_v2.py"]

model = dict(
    type="SemiSLClassifier",
    task="classification",
    backbone=dict(
        version="s_21k",
    ),
    head=dict(type="SemiLinearClsHead", loss=dict(type="CrossEntropyLoss", loss_weight=1.0)),
)

fp16 = dict(loss_scale=512.0)



"""This module tests classes related to Subset"""

# INTEL CONFIDENTIAL
#
# Copyright (C) 2021 Intel Corporation
#
# This software and the related documents are Intel copyrighted materials, and
# your use of them is governed by the express license under which they were provided to
# you ("License"). Unless the License provides otherwise, you may not use, modify, copy,
# publish, distribute, disclose or transmit this software or the related documents
# without Intel's prior written permission.
#
# This software and the related documents are provided as is,
# with no express or implied warranties, other than those that are expressly stated
# in the License.

import pytest

from otx.api.entities.subset import Subset
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestSubset:
    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_subset_members(self):
        """
        <b>Description:</b>
        To test Subset enumeration members

        <b>Input data:</b>
        Initialized instance of Subset enum

        <b>Expected results:</b>
        Enum members return correct values:

        NONE = 0
        TRAINING = 1
        VALIDATION = 2
        TESTING = 3
        UNLABELED = 4
        PSEUDOLABELED = 5
        UNASSIGNED = 6

        <b>Steps</b>
        1. Create enum instance
        2. Check members
        """
        test_instance = Subset

        for i in range(0, 7):
            assert test_instance(i) in list(Subset)

        with pytest.raises(AttributeError):
            test_instance.WRONG

        with pytest.raises(ValueError):
            test_instance(7)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_subset_magic_str(self):
        """
        <b>Description:</b>
        To test Subset __str__ method

        <b>Input data:</b>
        Initialized instance of Subset enum

        <b>Expected results:</b>
        __str__ return correct string for every enum member
        In case incorrect member it raises attribute exception

        <b>Steps</b>
        1. Create enum instance
        2. Check returning value of __str__ method
        """
        test_instance = Subset
        magic_str_list = [str(i) for i in list(Subset)]

        for i in range(0, 7):
            assert str(test_instance(i)) in magic_str_list

        with pytest.raises(AttributeError):
            str(test_instance.WRONG)

        with pytest.raises(ValueError):
            str(test_instance(7))

        assert len(set(magic_str_list)) == len(magic_str_list)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_subset_magic_repr(self):
        """
        <b>Description:</b>
        To test Subset __repr__ method

        <b>Input data:</b>
        Initialized instance of Subset enum

        <b>Expected results:</b>
        __repr__ method returns correct string

        <b>Steps</b>
        1. Create enum instance
        2. Check returning value of magic methods
        """
        test_instance = Subset
        magic_repr_list = [repr(i) for i in list(Subset)]

        for i in range(0, 7):
            assert repr(test_instance(i)) in magic_repr_list

        with pytest.raises(AttributeError):
            repr(test_instance.WRONG)

        with pytest.raises(ValueError):
            repr(test_instance(7))

        assert len(set(magic_repr_list)) == len(magic_repr_list)



import os

import pytest

from otx.algorithms.common.adapters.mmcv.tasks.exporter import Exporter
from otx.algorithms.common.adapters.mmcv.utils.config_utils import MPAConfig
from otx.algorithms.common.adapters.mmdeploy.apis import NaiveExporter
from otx.algorithms.detection.adapters.mmdet.utils.builder import build_detector
from otx.algorithms.detection.adapters.mmdet.utils.exporter import DetectionExporter
from tests.test_suite.e2e_test_system import e2e_pytest_unit
from tests.unit.algorithms.detection.test_helpers import (
    DEFAULT_DET_RECIPE_CONFIG_PATH,
    DEFAULT_DET_TEMPLATE_DIR,
    DEFAULT_ISEG_RECIPE_CONFIG_PATH,
    DEFAULT_ISEG_TEMPLATE_DIR,
)


@e2e_pytest_unit
@pytest.mark.parametrize(
    "recipe_cfg, template_dir",
    [
        (DEFAULT_DET_RECIPE_CONFIG_PATH, DEFAULT_DET_TEMPLATE_DIR),
        (DEFAULT_ISEG_RECIPE_CONFIG_PATH, DEFAULT_ISEG_TEMPLATE_DIR),
    ],
)
def test_run(recipe_cfg, template_dir, mocker):
    exporter = DetectionExporter()
    model_cfg = MPAConfig.fromfile(os.path.join(template_dir, "model.py"))
    model_cfg.work_dir = "/tmp/"
    args = {"precision": "FP32", "model_builder": build_detector}
    mocker.patch.object(Exporter, "run", return_value=True)
    returned_value = exporter.run(model_cfg)
    assert "model_builder" in args
    assert returned_value is True


@e2e_pytest_unit
@pytest.mark.parametrize(
    "recipe_cfg, template_dir",
    [
        (DEFAULT_DET_RECIPE_CONFIG_PATH, DEFAULT_DET_TEMPLATE_DIR),
        (DEFAULT_ISEG_RECIPE_CONFIG_PATH, DEFAULT_ISEG_TEMPLATE_DIR),
    ],
)
def test_naive_export(recipe_cfg, template_dir, mocker):
    exporter = DetectionExporter()
    data_cfg = MPAConfig.fromfile(os.path.join(template_dir, "data_pipeline.py"))
    mock_export_ov = mocker.patch.object(NaiveExporter, "export2openvino")
    exporter.naive_export("", build_detector, "FP32", data_cfg)
    mock_export_ov.assert_called_once()



# Copyright (C) 2021-2022 Intel Corporation
# SPDX-License-Identifier: Apache-2.0
#

import numpy as np
import pytest

from otx.api.entities.tensor import TensorEntity
from tests.unit.api.constants.components import OtxSdkComponent
from tests.unit.api.constants.requirements import Requirements

RANDOM_NUMPY = np.random.randint(low=0, high=255, size=(16, 32, 3))


@pytest.mark.components(OtxSdkComponent.OTX_API)
class TestTensorEntity:
    @staticmethod
    def tensor_params():
        return {"name": "Test Tensor", "numpy": RANDOM_NUMPY}

    def tensor(self):
        return TensorEntity(**self.tensor_params())

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_tensor_initialization(self):
        """
        <b>Description:</b>
        Check TensorEntity class object initialization

        <b>Input data:</b>
        TensorEntity class object with specified "name" and "numpy" parameters

        <b>Expected results:</b>
        Test passes if attributes of initialized TensorEntity class object are equal to expected
        """
        tensor_params = self.tensor_params()
        tensor = TensorEntity(**tensor_params)
        assert tensor.name == tensor_params.get("name")
        assert np.array_equal(tensor.numpy, tensor_params.get("numpy"))

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_tensor_shape(self):
        """
        <b>Description:</b>
        Check TensorEntity class object "shape" property

        <b>Input data:</b>
        TensorEntity class object with specified "name" and "numpy" parameters

        <b>Expected results:</b>
        Test passes if value returned by "shape" property is equal to expected

        <b>Steps</b>
        1. Check value returned by "shape" property for initialized TensorEntity object
        2. Manually set new value of "numpy" property and check re-check "numpy" and "shape" properties
        """
        # Checking values returned by "shape" property for initialized TensorEntity object
        tensor = self.tensor()
        assert tensor.shape == (16, 32, 3)
        # Manually setting new value of "numpy" property and re-checking "numpy and "shape" properties
        new_numpy = np.random.uniform(low=0.0, high=255.0, size=(8, 16, 3))
        tensor.numpy = new_numpy
        assert np.array_equal(tensor.numpy, new_numpy)
        assert tensor.shape == (8, 16, 3)

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_tensor_eq(self):
        """
        <b>Description:</b>
        Check TensorEntity class object __eq__ method

        <b>Input data:</b>
        TensorEntity class objects with specified "name" and "numpy" parameters

        <b>Expected results:</b>
        Test passes if value returned by __eq__ method is equal to expected

        <b>Steps</b>
        1. Check value returned by __eq__ method for comparing equal TensorEntity objects
        2. Check value returned by __eq__ method for comparing TensorEntity objects with unequal "name" parameters:
        expected equality
        3. Check value returned by __eq__ method for comparing TensorEntity objects with unequal "numpy" parameters -
        expected inequality
        4. Check value returned by __eq__ method for comparing TensorEntity with different type object
        """
        initialization_params = self.tensor_params()
        tensor = TensorEntity(**initialization_params)
        # Comparing equal TensorEntity objects
        equal_tensor = TensorEntity(**initialization_params)
        assert tensor == equal_tensor
        # Comparing TensorEntity objects with unequal "name" parameter, expected equality
        unequal_params = dict(initialization_params)
        unequal_params["name"] = "Unequal_name"
        equal_tensor = TensorEntity(**unequal_params)
        assert tensor == equal_tensor
        # Comparing TensorEntity objects with unequal "numpy" parameter, expected inequality
        unequal_params = dict(initialization_params)
        unequal_params["numpy"] = np.random.uniform(low=0.0, high=255.0, size=(1, 2, 3))
        unequal_tensor = TensorEntity(**unequal_params)
        assert tensor != unequal_tensor
        # Comparing TensorEntity with different type object
        assert tensor != str

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_tensor_str(self):
        """
        <b>Description:</b>
        Check TensorEntity class object __str__ method

        <b>Input data:</b>
        TensorEntity class object with specified "name" and "numpy" parameters

        <b>Expected results:</b>
        Test passes if value returned by __str__ method is equal to expected
        """
        tensor = self.tensor()
        assert str(tensor) == "TensorEntity(name=Test Tensor, shape=(16, 32, 3))"

    @pytest.mark.priority_medium
    @pytest.mark.unit
    @pytest.mark.reqids(Requirements.REQ_1)
    def test_tensor_repr(self):
        """
        <b>Description:</b>
        Check TensorEntity class object __repr__ method

        <b>Input data:</b>
        TensorEntity class object with specified "name" and "numpy" parameters

        <b>Expected results:</b>
        Test passes if value returned by __repr__ method is equal to expected
        """
        tensor = self.tensor()
        assert repr(tensor) == "TensorEntity(name=Test Tensor)"
